class FileStream(Stream,IDisposable):
 """
 Exposes a System.IO.Stream around a file,supporting both synchronous and asynchronous read and write operations.

 

 FileStream(path: str,mode: FileMode)

 FileStream(path: str,mode: FileMode,access: FileAccess)

 FileStream(path: str,mode: FileMode,access: FileAccess,share: FileShare)

 FileStream(path: str,mode: FileMode,access: FileAccess,share: FileShare,bufferSize: int)

 FileStream(path: str,mode: FileMode,access: FileAccess,share: FileShare,bufferSize: int,options: FileOptions)

 FileStream(path: str,mode: FileMode,access: FileAccess,share: FileShare,bufferSize: int,useAsync: bool)

 FileStream(path: str,mode: FileMode,rights: FileSystemRights,share: FileShare,bufferSize: int,options: FileOptions,fileSecurity: FileSecurity)

 FileStream(path: str,mode: FileMode,rights: FileSystemRights,share: FileShare,bufferSize: int,options: FileOptions)

 FileStream(handle: IntPtr,access: FileAccess)

 FileStream(handle: IntPtr,access: FileAccess,ownsHandle: bool)

 FileStream(handle: IntPtr,access: FileAccess,ownsHandle: bool,bufferSize: int)

 FileStream(handle: IntPtr,access: FileAccess,ownsHandle: bool,bufferSize: int,isAsync: bool)

 FileStream(handle: SafeFileHandle,access: FileAccess)

 FileStream(handle: SafeFileHandle,access: FileAccess,bufferSize: int)

 FileStream(handle: SafeFileHandle,access: FileAccess,bufferSize: int,isAsync: bool)
 """
 def BeginRead(self,array,offset,numBytes,userCallback,stateObject):
  """
  BeginRead(self: FileStream,array: Array[Byte],offset: int,numBytes: int,userCallback: AsyncCallback,stateObject: object) -> IAsyncResult

  

   Begins an asynchronous read.

  

   array: The buffer to read data into.

   offset: The byte offset in array at which to begin reading.

   numBytes: The maximum number of bytes to read.

   userCallback: The method to be called when the asynchronous read operation is completed.

   stateObject: A user-provided object that distinguishes this particular asynchronous read request from other 

    requests.

  

   Returns: An object that references the asynchronous read.
  """
  pass
 def BeginWrite(self,array,offset,numBytes,userCallback,stateObject):
  """
  BeginWrite(self: FileStream,array: Array[Byte],offset: int,numBytes: int,userCallback: AsyncCallback,stateObject: object) -> IAsyncResult

  

   Begins an asynchronous write.

  

   array: The buffer containing data to write to the current stream.

   offset: The zero-based byte offset in array at which to begin copying bytes to the current stream.

   numBytes: The maximum number of bytes to write.

   userCallback: The method to be called when the asynchronous write operation is completed.

   stateObject: A user-provided object that distinguishes this particular asynchronous write request from other 

    requests.

  

   Returns: An object that references the asynchronous write.
  """
  pass
 def CreateWaitHandle(self,*args):
  """
  CreateWaitHandle(self: Stream) -> WaitHandle

  

   Allocates a System.Threading.WaitHandle object.

   Returns: A reference to the allocated WaitHandle.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: FileStream,disposing: bool)

   Releases the unmanaged resources used by the System.IO.FileStream and optionally releases the 

    managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def EndRead(self,asyncResult):
  """
  EndRead(self: FileStream,asyncResult: IAsyncResult) -> int

  

   Waits for the pending asynchronous read to complete.

  

   asyncResult: The reference to the pending asynchronous request to wait for.

   Returns: The number of bytes read from the stream,between 0 and the number of bytes you requested. 

    Streams only return 0 at the end of the stream,otherwise,they should block until at least 1 

    byte is available.
  """
  pass
 def EndWrite(self,asyncResult):
  """
  EndWrite(self: FileStream,asyncResult: IAsyncResult)

   Ends an asynchronous write,blocking until the I/O operation has completed.

  

   asyncResult: The pending asynchronous I/O request.
  """
  pass
 def Flush(self,flushToDisk=None):
  """
  Flush(self: FileStream,flushToDisk: bool)

   Clears buffers for this stream and causes any buffered data to be written to the file,and also 

    clears all intermediate file buffers.

  

  

   flushToDisk: true to flush all intermediate file buffers; otherwise,false.

  Flush(self: FileStream)

   Clears buffers for this stream and causes any buffered data to be written to the file.
  """
  pass
 def FlushAsync(self,cancellationToken=None):
  """ FlushAsync(self: FileStream,cancellationToken: CancellationToken) -> Task """
  pass
 def GetAccessControl(self):
  """
  GetAccessControl(self: FileStream) -> FileSecurity

  

   Gets a System.Security.AccessControl.FileSecurity object that encapsulates the access control 

    list (ACL) entries for the file described by the current System.IO.FileStream object.

  

   Returns: An object that encapsulates the access control settings for the file described by the current 

    System.IO.FileStream object.
  """
  pass
 def Lock(self,position,length):
  """
  Lock(self: FileStream,position: Int64,length: Int64)

   Prevents other processes from reading from or writing to the System.IO.FileStream.

  

   position: The beginning of the range to lock. The value of this parameter must be equal to or greater than 

    zero (0).

  

   length: The range to be locked.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def ObjectInvariant(self,*args):
  """
  ObjectInvariant(self: Stream)

   Provides support for a System.Diagnostics.Contracts.Contract.
  """
  pass
 def Read(self,array,offset,count):
  """
  Read(self: FileStream,offset: int,count: int) -> (int,Array[Byte])

  

   Reads a block of bytes from the stream and writes the data in a given buffer.

  

   offset: The byte offset in array at which the read bytes will be placed.

   count: The maximum number of bytes to read.

   Returns: The total number of bytes read into the buffer. This might be less than the number of bytes 

    requested if that number of bytes are not currently available,or zero if the end of the stream 

    is reached.
  """
  pass
 def ReadAsync(self,buffer,offset,count,cancellationToken=None):
  """ ReadAsync(self: FileStream,buffer: Array[Byte],offset: int,count: int,cancellationToken: CancellationToken) -> Task[int] """
  pass
 def ReadByte(self):
  """
  ReadByte(self: FileStream) -> int

  

   Reads a byte from the file and advances the read position one byte.

   Returns: The byte,cast to an System.Int32,or -1 if the end of the stream has been reached.
  """
  pass
 def Seek(self,offset,origin):
  """
  Seek(self: FileStream,offset: Int64,origin: SeekOrigin) -> Int64

  

   Sets the current position of this stream to the given value.

  

   offset: The point relative to origin from which to begin seeking.

   origin: Specifies the beginning,the end,or the current position as a reference point for origin,using 

    a value of type System.IO.SeekOrigin.

  

   Returns: The new position in the stream.
  """
  pass
 def SetAccessControl(self,fileSecurity):
  """
  SetAccessControl(self: FileStream,fileSecurity: FileSecurity)

   Applies access control list (ACL) entries described by a 

    System.Security.AccessControl.FileSecurity object to the file described by the current 

    System.IO.FileStream object.

  

  

   fileSecurity: An object that describes an ACL entry to apply to the current file.
  """
  pass
 def SetLength(self,value):
  """
  SetLength(self: FileStream,value: Int64)

   Sets the length of this stream to the given value.

  

   value: The new length of the stream.
  """
  pass
 def Unlock(self,position,length):
  """
  Unlock(self: FileStream,position: Int64,length: Int64)

   Allows access by other processes to all or part of a file that was previously locked.

  

   position: The beginning of the range to unlock.

   length: The range to be unlocked.
  """
  pass
 def Write(self,array,offset,count):
  """
  Write(self: FileStream,array: Array[Byte],offset: int,count: int)

   Writes a block of bytes to this stream using data from a buffer.

  

   array: The buffer containing data to write to the stream.

   offset: The zero-based byte offset in array at which to begin copying bytes to the current stream.

   count: The number of bytes to be written to the current stream.
  """
  pass
 def WriteAsync(self,buffer,offset,count,cancellationToken=None):
  """ WriteAsync(self: FileStream,buffer: Array[Byte],offset: int,count: int,cancellationToken: CancellationToken) -> Task """
  pass
 def WriteByte(self,value):
  """
  WriteByte(self: FileStream,value: Byte)

   Writes a byte to the current position in the file stream.

  

   value: A byte to write to the stream.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,path: str,mode: FileMode)

  __new__(cls: type,path: str,mode: FileMode,access: FileAccess)

  __new__(cls: type,path: str,mode: FileMode,access: FileAccess,share: FileShare)

  __new__(cls: type,path: str,mode: FileMode,access: FileAccess,share: FileShare,bufferSize: int)

  __new__(cls: type,path: str,mode: FileMode,access: FileAccess,share: FileShare,bufferSize: int,options: FileOptions)

  __new__(cls: type,path: str,mode: FileMode,access: FileAccess,share: FileShare,bufferSize: int,useAsync: bool)

  __new__(cls: type,path: str,mode: FileMode,rights: FileSystemRights,share: FileShare,bufferSize: int,options: FileOptions,fileSecurity: FileSecurity)

  __new__(cls: type,path: str,mode: FileMode,rights: FileSystemRights,share: FileShare,bufferSize: int,options: FileOptions)

  __new__(cls: type,handle: IntPtr,access: FileAccess)

  __new__(cls: type,handle: IntPtr,access: FileAccess,ownsHandle: bool)

  __new__(cls: type,handle: IntPtr,access: FileAccess,ownsHandle: bool,bufferSize: int)

  __new__(cls: type,handle: IntPtr,access: FileAccess,ownsHandle: bool,bufferSize: int,isAsync: bool)

  __new__(cls: type,handle: SafeFileHandle,access: FileAccess)

  __new__(cls: type,handle: SafeFileHandle,access: FileAccess,bufferSize: int)

  __new__(cls: type,handle: SafeFileHandle,access: FileAccess,bufferSize: int,isAsync: bool)
  """
  pass
 CanRead=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the current stream supports reading.



Get: CanRead(self: FileStream) -> bool



"""

 CanSeek=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the current stream supports seeking.



Get: CanSeek(self: FileStream) -> bool



"""

 CanWrite=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the current stream supports writing.



Get: CanWrite(self: FileStream) -> bool



"""

 Handle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the operating system file handle for the file that the current FileStream object encapsulates.



Get: Handle(self: FileStream) -> IntPtr



"""

 IsAsync=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the FileStream was opened asynchronously or synchronously.



Get: IsAsync(self: FileStream) -> bool



"""

 Length=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the length in bytes of the stream.



Get: Length(self: FileStream) -> Int64



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the FileStream that was passed to the constructor.



Get: Name(self: FileStream) -> str



"""

 Position=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the current position of this stream.



Get: Position(self: FileStream) -> Int64



Set: Position(self: FileStream)=value

"""

 SafeFileHandle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a Microsoft.Win32.SafeHandles.SafeFileHandle object that represents the operating system file handle for the file that the current System.IO.FileStream object encapsulates.



Get: SafeFileHandle(self: FileStream) -> SafeFileHandle



"""





class HtmlDocument(object):
 """ Provides top-level programmatic access to an HTML document hosted by the System.Windows.Forms.WebBrowser control. """
 def AttachEventHandler(self,eventName,eventHandler):
  """
  AttachEventHandler(self: HtmlDocument,eventName: str,eventHandler: EventHandler)

   Adds an event handler for the named HTML DOM event.

  

   eventName: The name of the event you want to handle.

   eventHandler: The managed code that handles the event.
  """
  pass
 def CreateElement(self,elementTag):
  """
  CreateElement(self: HtmlDocument,elementTag: str) -> HtmlElement

  

   Creates a new HtmlElement of the specified HTML tag type.

  

   elementTag: The name of the HTML element to create.

   Returns: A new element of the specified tag type.
  """
  pass
 def DetachEventHandler(self,eventName,eventHandler):
  """
  DetachEventHandler(self: HtmlDocument,eventName: str,eventHandler: EventHandler)

   Removes an event handler from a named event on the HTML DOM.

  

   eventName: The name of the event you want to cease handling.

   eventHandler: The managed code that handles the event.
  """
  pass
 def Equals(self,obj):
  """
  Equals(self: HtmlDocument,obj: object) -> bool

  

   obj: The object to compare with the current object.

   Returns: true if the specified System.Object is equal to the current System.Object; otherwise,false.
  """
  pass
 def ExecCommand(self,command,showUI,value):
  """
  ExecCommand(self: HtmlDocument,command: str,showUI: bool,value: object)

   Executes the specified command against the document.

  

   command: The name of the command to execute.

   showUI: Whether or not to show command-specific dialog boxes or message boxes to the user.

   value: The value to assign using the command. Not applicable for all commands.
  """
  pass
 def Focus(self):
  """
  Focus(self: HtmlDocument)

   Sets user input focus on the current document.
  """
  pass
 def GetElementById(self,id):
  """
  GetElementById(self: HtmlDocument,id: str) -> HtmlElement

  

   Retrieves a single System.Windows.Forms.HtmlElement using the element's ID attribute as a search 

    key.

  

  

   id: The ID attribute of the element to retrieve.

   Returns: Returns the first object with the same ID attribute as the specified value,or null if the id 

    cannot be found.
  """
  pass
 def GetElementFromPoint(self,point):
  """
  GetElementFromPoint(self: HtmlDocument,point: Point) -> HtmlElement

  

   Retrieves the HTML element located at the specified client coordinates.

  

   point: The x,y position of the element on the screen,relative to the top-left corner of the document.

   Returns: The System.Windows.Forms.HtmlElement at the specified screen location in the document.
  """
  pass
 def GetElementsByTagName(self,tagName):
  """
  GetElementsByTagName(self: HtmlDocument,tagName: str) -> HtmlElementCollection

  

   Retrieve a collection of elements with the specified HTML tag.

  

   tagName: The name of the HTML tag for the System.Windows.Forms.HtmlElement objects you want to retrieve.

   Returns: The collection of elements who tag name is equal to the tagName argument.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: HtmlDocument) -> int

   Returns: A hash code for the current System.Object.
  """
  pass
 def InvokeScript(self,scriptName,args=None):
  """
  InvokeScript(self: HtmlDocument,scriptName: str) -> object

  

   Executes an Active Scripting function defined in an HTML page.

  

   scriptName: The name of the script method to invoke.

   Returns: The object returned by the Active Scripting call.

  InvokeScript(self: HtmlDocument,scriptName: str,args: Array[object]) -> object

  

   Executes an Active Scripting function defined in an HTML page.

  

   scriptName: The name of the script method to invoke.

   args: The arguments to pass to the script method.

   Returns: The object returned by the Active Scripting call.
  """
  pass
 def OpenNew(self,replaceInHistory):
  """
  OpenNew(self: HtmlDocument,replaceInHistory: bool) -> HtmlDocument

  

   Gets a new System.Windows.Forms.HtmlDocument to use with the 

    System.Windows.Forms.HtmlDocument.Write(System.String) method.

  

  

   replaceInHistory: Whether the new window's navigation should replace the previous element in the navigation 

    history of the DOM.

  

   Returns: A new document for writing.
  """
  pass
 def Write(self,text):
  """
  Write(self: HtmlDocument,text: str)

   Writes a new HTML page.

  

   text: The HTML text to write into the document.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __ne__(self,*args):
  pass
 ActiveElement=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides the System.Windows.Forms.HtmlElement which currently has user input focus.



Get: ActiveElement(self: HtmlDocument) -> HtmlElement



"""

 ActiveLinkColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Drawing.Color of a hyperlink when clicked by a user.



Get: ActiveLinkColor(self: HtmlDocument) -> Color



Set: ActiveLinkColor(self: HtmlDocument)=value

"""

 All=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an instance of System.Windows.Forms.HtmlElementCollection,which stores all System.Windows.Forms.HtmlElement objects for the document.



Get: All(self: HtmlDocument) -> HtmlElementCollection



"""

 BackColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the background color of the HTML document.



Get: BackColor(self: HtmlDocument) -> Color



Set: BackColor(self: HtmlDocument)=value

"""

 Body=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.HtmlElement for the BODY tag.



Get: Body(self: HtmlDocument) -> HtmlElement



"""

 Cookie=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the HTTP cookies associated with this document.



Get: Cookie(self: HtmlDocument) -> str



Set: Cookie(self: HtmlDocument)=value

"""

 DefaultEncoding=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the encoding used by default for the current document.



Get: DefaultEncoding(self: HtmlDocument) -> str



"""

 Domain=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the string describing the domain of this document for security purposes.



Get: Domain(self: HtmlDocument) -> str



Set: Domain(self: HtmlDocument)=value

"""

 DomDocument=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the unmanaged interface pointer for this System.Windows.Forms.HtmlDocument.



Get: DomDocument(self: HtmlDocument) -> object



"""

 Encoding=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the character encoding for this document.



Get: Encoding(self: HtmlDocument) -> str



Set: Encoding(self: HtmlDocument)=value

"""

 Focused=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the document has user input focus.



Get: Focused(self: HtmlDocument) -> bool



"""

 ForeColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the text color for the document.



Get: ForeColor(self: HtmlDocument) -> Color



Set: ForeColor(self: HtmlDocument)=value

"""

 Forms=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a collection of all of the <FORM> elements in the document.



Get: Forms(self: HtmlDocument) -> HtmlElementCollection



"""

 Images=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a collection of all image tags in the document.



Get: Images(self: HtmlDocument) -> HtmlElementCollection



"""

 LinkColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the color of hyperlinks.



Get: LinkColor(self: HtmlDocument) -> Color



Set: LinkColor(self: HtmlDocument)=value

"""

 Links=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a list of all the hyperlinks within this HTML document.



Get: Links(self: HtmlDocument) -> HtmlElementCollection



"""

 RightToLeft=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the direction of text in the current document.



Get: RightToLeft(self: HtmlDocument) -> bool



Set: RightToLeft(self: HtmlDocument)=value

"""

 Title=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the text value of the <TITLE> tag in the current HTML document.



Get: Title(self: HtmlDocument) -> str



Set: Title(self: HtmlDocument)=value

"""

 Url=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the URL describing the location of this document.



Get: Url(self: HtmlDocument) -> Uri



"""

 VisitedLinkColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the Color of links to HTML pages that the user has already visited.



Get: VisitedLinkColor(self: HtmlDocument) -> Color



Set: VisitedLinkColor(self: HtmlDocument)=value

"""

 Window=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.HtmlWindow associated with this document.



Get: Window(self: HtmlDocument) -> HtmlWindow



"""


 Click=None
 ContextMenuShowing=None
 Focusing=None
 LosingFocus=None
 MouseDown=None
 MouseLeave=None
 MouseMove=None
 MouseOver=None
 MouseUp=None
 Stop=None




"""Handle updating pyRevit repository and its extensions."""

import pyrevit.coreutils.git as libgit
from pyrevit.compat import safe_strtype
from pyrevit import coreutils
from pyrevit import HOME_DIR
from pyrevit.coreutils.logger import get_logger
from pyrevit import versionmgr
from pyrevit.versionmgr import upgrade

from pyrevit.userconfig import user_config
from pyrevit.extensions import extensionmgr

import socket

#pylint: disable=C0103,W0703
logger = get_logger(__name__)


COREUPDATE_TRIGGER = 'COREUPDATE'

COREUPDATE_MESSAGE = '<div class="coreupdatewarn">' \
                     '<strong>IMPORTANT</strong>\n' \
                     'pyRevit has a major core update. This update ' \
                     '<u>can not</u> be applied when Revit is running. ' \
                     'Please close all Revit instances, and update the clone ' \
                     'using the pyRevit CLI. Start Revit ' \
                     'again after the update and pyRevit will load with ' \
                     'the new core changes.' \
                     '</div>'


def _check_connection(host="8.8.8.8", port=53, timeout=3):
    try:
        socket.setdefaulttimeout(timeout)
        socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect((host, port))
        logger.debug('Internet access detected.')
        return True
    except socket.error as ex:
        logger.debug('No internet access detected. %s', ex)
        return False


def _get_extension_credentials(repo_info):
    try:
        repo_config = user_config.get_section(repo_info.name)
        if repo_config.private_repo:
            return repo_config.username, repo_config.password
        return None, None
    except Exception:
        return None, None


def _fetch_remote(remote, repo_info):
    logger.debug('Fetching remote: %s of %s', remote.Name, repo_info.directory)
    username, password = _get_extension_credentials(repo_info)
    if username and password:
        repo_info.username = username
        repo_info.password = password

    libgit.git_fetch(repo_info)


def get_thirdparty_ext_repos():
    """Return a list of repos for installed third-party extensions."""
    processed_paths = set()
    valid_exts = []
    ext_repos = []
    logger.debug('Finding installed repos...')
    ext_info_list = extensionmgr.get_thirdparty_extension_data()

    for ext_info in ext_info_list:
        repo_path = libgit.libgit.Repository.Discover(ext_info.directory)
        if repo_path:
            repo_info = libgit.get_repo(repo_path)
            if repo_info:
                valid_exts.append(ext_info)
                if repo_info.directory not in processed_paths:
                    processed_paths.add(repo_info.directory)
                    ext_repos.append(repo_info)

    logger.debug('Valid third-party extensions for update: %s', valid_exts)

    return ext_repos


def get_all_extension_repos():
    """Return a list of repos for all installed extensions."""
    logger.debug('Finding all extension repos.')
    # pyrevit main repo
    repo_info_list = []
    pyrevit_repo = versionmgr.get_pyrevit_repo()
    if pyrevit_repo:
        repo_info_list.append(pyrevit_repo)
    # add all thirdparty extension repos
    repo_info_list.extend(get_thirdparty_ext_repos())
    logger.debug('Repos are: %s', repo_info_list)
    return repo_info_list


def update_repo(repo_info):
    """Update repository.

    Args:
        repo_info (:obj:`pyrevit.coreutils.git.RepoInfo`):
            repository info wrapper object
    """
    repo = repo_info.repo
    logger.debug('Updating repo: %s', repo_info.directory)
    head_msg = safe_strtype(repo.Head.Tip.Message).replace('\n', '')
    logger.debug('Current head is: %s > %s', repo.Head.Tip.Id.Sha, head_msg)
    username, password = _get_extension_credentials(repo_info)
    if username and password:
        repo_info.username = username
        repo_info.password = password

    try:
        updated_repo_info = libgit.git_pull(repo_info)
        logger.debug('Successfully updated repo: %s',
                     updated_repo_info.directory)
        return updated_repo_info

    except libgit.PyRevitGitAuthenticationError as auth_err:
        logger.debug('Can not login to git repository to get updates: %s | %s',
                     repo_info, auth_err)
        raise auth_err

    except Exception as update_err:
        logger.debug('Failed updating repo: %s | %s', repo_info, update_err)
        raise update_err


def get_updates(repo_info):
    """Fetch updates on repository.

    Args:
        repo_info (:obj:`pyrevit.coreutils.git.RepoInfo`):
            repository info wrapper object
    """
    repo = repo_info.repo
    at_least_one_fetch_was_successful = False

    logger.debug('Fetching updates for: %s', repo_info.directory)
    for remote in repo.Network.Remotes:
        try:
            _fetch_remote(remote, repo_info)
            at_least_one_fetch_was_successful = True

        except libgit.PyRevitGitAuthenticationError:
            logger.debug('Failed fetching updates. '
                         'Can not login to repo to get updates: %s', repo_info)
            continue

        except Exception:
            logger.debug('Failed fetching updates: %s', repo_info)
            continue

    if at_least_one_fetch_was_successful:
        return True

    return False


def has_pending_updates(repo_info):
    """Check for updates on repository.

    Args:
        repo_info (:obj:`pyrevit.coreutils.git.RepoInfo`):
            repository info wrapper object
    """
    if get_updates(repo_info):
        hist_div = libgit.compare_branch_heads(repo_info)
        if hist_div.BehindBy > 0:
            return True


def check_for_updates():
    """Check whether any available repo has pending updates."""
    if _check_connection():
        logger.info('Checking for updates...')

        for repo in get_all_extension_repos():
            if has_pending_updates(repo):
                logger.info('Updates are available for %s...', repo.name)
                return True
            else:
                logger.info('%s is up-to-date...', repo.name)
    else:
        logger.warning('No internet access detected. '
                       'Skipping check for updates.')
        return False


def has_core_updates():
    """Check whether pyRevit repo has core updates.

    This would require host application to be closed to release the file lock
    of core DLLs so they can be updated separately.
    """
    pyrevit_repo = versionmgr.get_pyrevit_repo()
    if pyrevit_repo and get_updates(pyrevit_repo):
        new_commits = libgit.get_all_new_commits(pyrevit_repo)

        logger.debug('Checking new commits on pyrevit repo.')
        for cmt_sha, cmt_msg in new_commits.items():
            logger.debug('%s: %s', cmt_sha, cmt_msg)
            if COREUPDATE_TRIGGER in cmt_msg:
                logger.debug('pyrevit repo has core update at %s: %s',
                             cmt_sha, cmt_msg)
                return True

    return False


def update_pyrevit():
    """Update pyrevit and its extension repositories."""
    if _check_connection():
        third_party_updated = False
        pyrevit_updated = False
        pyrevit_has_coreupdates = has_core_updates()
        thirdparty_repos = get_thirdparty_ext_repos()

        logger.debug('List of thirdparty repos to be updated: %s',
                     thirdparty_repos)

        # update third-party extensions first, one by one
        for repo_info in thirdparty_repos:
            logger.debug('Updating repo: %s', repo_info.directory)
            try:
                upped_repo_info = update_repo(repo_info)
                logger.info(':inbox_tray: Successfully updated: %s to %s',
                            upped_repo_info.name,
                            upped_repo_info.last_commit_hash[:7])
                third_party_updated = True
            except Exception:
                logger.info('Can not update repo: %s (Run in debug to see why)',
                            repo_info.name)

        # now update pyrevit repo and reload
        pyrevit_repo = versionmgr.get_pyrevit_repo()
        if pyrevit_repo:
            if not pyrevit_has_coreupdates:
                logger.debug('Updating pyrevit repo: %s',
                             pyrevit_repo.directory)
                try:
                    upped_pyrevit_repo_info = update_repo(pyrevit_repo)
                    logger.info(':inbox_tray: Successfully updated: %s to %s',
                                upped_pyrevit_repo_info.name,
                                upped_pyrevit_repo_info.last_commit_hash[:7])
                    pyrevit_updated = True
                except Exception as err:
                    logger.info('Can not update pyrevit repo '
                                '(Run in debug to see why) | %s', err)
                # perform upgrade tasks
                logger.info('Upgrading settings...')
                upgrade.upgrade_existing_pyrevit()
        if not pyrevit_has_coreupdates:
            if third_party_updated or pyrevit_updated:
                # now reload pyrevit
                from pyrevit.loader import sessionmgr
                sessionmgr.reload_pyrevit()
            else:
                logger.info('pyRevit and extensions seem to be up-to-date.')
        else:
            from pyrevit import script
            output = script.get_output()
            output.print_html(COREUPDATE_MESSAGE)
            logger.debug('Core updates. Skippin update and reload.')
    else:
        logger.warning('No internet access detected. Skipping update.')



# encoding: utf-8
# module System.Windows.Automation.Peers calls itself Peers
# from PresentationCore,Version=4.0.0.0,Culture=neutral,PublicKeyToken=31bf3856ad364e35
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class AutomationControlType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the System.Windows.Automation.ControlType that is exposed to the UI Automation�client.

 

 enum AutomationControlType,values: Button (0),Calendar (1),CheckBox (2),ComboBox (3),Custom (25),DataGrid (28),DataItem (29),Document (30),Edit (4),Group (26),Header (34),HeaderItem (35),Hyperlink (5),Image (6),List (8),ListItem (7),Menu (9),MenuBar (10),MenuItem (11),Pane (33),ProgressBar (12),RadioButton (13),ScrollBar (14),Separator (38),Slider (15),Spinner (16),SplitButton (31),StatusBar (17),Tab (18),TabItem (19),Table (36),Text (20),Thumb (27),TitleBar (37),ToolBar (21),ToolTip (22),Tree (23),TreeItem (24),Window (32)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Button=None
 Calendar=None
 CheckBox=None
 ComboBox=None
 Custom=None
 DataGrid=None
 DataItem=None
 Document=None
 Edit=None
 Group=None
 Header=None
 HeaderItem=None
 Hyperlink=None
 Image=None
 List=None
 ListItem=None
 Menu=None
 MenuBar=None
 MenuItem=None
 Pane=None
 ProgressBar=None
 RadioButton=None
 ScrollBar=None
 Separator=None
 Slider=None
 Spinner=None
 SplitButton=None
 StatusBar=None
 Tab=None
 TabItem=None
 Table=None
 Text=None
 Thumb=None
 TitleBar=None
 ToolBar=None
 ToolTip=None
 Tree=None
 TreeItem=None
 value__=None
 Window=None


class AutomationEvents(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the event that is raised by the element through the associated System.Windows.Automation.Peers.AutomationPeer.

 

 enum AutomationEvents,values: AsyncContentLoaded (12),AutomationFocusChanged (4),InputDiscarded (17),InputReachedOtherElement (16),InputReachedTarget (15),InvokePatternOnInvoked (5),MenuClosed (3),MenuOpened (2),PropertyChanged (13),SelectionItemPatternOnElementAddedToSelection (6),SelectionItemPatternOnElementRemovedFromSelection (7),SelectionItemPatternOnElementSelected (8),SelectionPatternOnInvalidated (9),StructureChanged (14),TextPatternOnTextChanged (11),TextPatternOnTextSelectionChanged (10),ToolTipClosed (1),ToolTipOpened (0)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AsyncContentLoaded=None
 AutomationFocusChanged=None
 InputDiscarded=None
 InputReachedOtherElement=None
 InputReachedTarget=None
 InvokePatternOnInvoked=None
 MenuClosed=None
 MenuOpened=None
 PropertyChanged=None
 SelectionItemPatternOnElementAddedToSelection=None
 SelectionItemPatternOnElementRemovedFromSelection=None
 SelectionItemPatternOnElementSelected=None
 SelectionPatternOnInvalidated=None
 StructureChanged=None
 TextPatternOnTextChanged=None
 TextPatternOnTextSelectionChanged=None
 ToolTipClosed=None
 ToolTipOpened=None
 value__=None


class AutomationOrientation(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the direction in which a control can be laid out.

 

 enum AutomationOrientation,values: Horizontal (1),None (0),Vertical (2)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Horizontal=None
 None=None
 value__=None
 Vertical=None


class AutomationPeer(DispatcherObject):
 """ Provides a base class that exposes an element to UI Automation. """
 def GetAcceleratorKey(self):
  """
  GetAcceleratorKey(self: AutomationPeer) -> str

  

   Gets the accelerator key combinations for the element that is associated with the UI Automation 

    peer.

  

   Returns: The accelerator key.
  """
  pass
 def GetAcceleratorKeyCore(self,*args):
  """
  GetAcceleratorKeyCore(self: AutomationPeer) -> str

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetAcceleratorKey.

  

   Returns: The accelerator key.
  """
  pass
 def GetAccessKey(self):
  """
  GetAccessKey(self: AutomationPeer) -> str

  

   Gets the access key for the element that is associated with the automation peer.

   Returns: The string that contains the access key.
  """
  pass
 def GetAccessKeyCore(self,*args):
  """
  GetAccessKeyCore(self: AutomationPeer) -> str

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetAccessKey.

  

   Returns: The string that contains the access key.
  """
  pass
 def GetAutomationControlType(self):
  """
  GetAutomationControlType(self: AutomationPeer) -> AutomationControlType

  

   Gets the control type for the element that is associated with the UI Automation peer.

   Returns: The control type.
  """
  pass
 def GetAutomationControlTypeCore(self,*args):
  """
  GetAutomationControlTypeCore(self: AutomationPeer) -> AutomationControlType

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetAutomationControlType.

  

   Returns: The control type.
  """
  pass
 def GetAutomationId(self):
  """
  GetAutomationId(self: AutomationPeer) -> str

  

   Gets the System.Windows.Automation.AutomationProperties.AutomationId of the element that is 

    associated with the automation peer.

  

   Returns: The identifier.
  """
  pass
 def GetAutomationIdCore(self,*args):
  """
  GetAutomationIdCore(self: AutomationPeer) -> str

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetAutomationId.

  

   Returns: The string that contains the identifier.
  """
  pass
 def GetBoundingRectangle(self):
  """
  GetBoundingRectangle(self: AutomationPeer) -> Rect

  

   Gets the System.Windows.Rect object that represents the screen coordinates of the element that 

    is associated with the automation peer.

  

   Returns: The bounding rectangle.
  """
  pass
 def GetBoundingRectangleCore(self,*args):
  """
  GetBoundingRectangleCore(self: AutomationPeer) -> Rect

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetBoundingRectangle.

  

   Returns: The bounding rectangle.
  """
  pass
 def GetChildren(self):
  """
  GetChildren(self: AutomationPeer) -> List[AutomationPeer]

  

   Gets the collection of System.Windows.Automation.Peers.AutomationPeer.GetChildren elements that 

    are represented in the UI Automation tree as immediate child elements of the automation peer.

  

   Returns: The collection of child elements.
  """
  pass
 def GetChildrenCore(self,*args):
  """
  GetChildrenCore(self: AutomationPeer) -> List[AutomationPeer]

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetChildren.

  

   Returns: The collection of child elements.
  """
  pass
 def GetClassName(self):
  """
  GetClassName(self: AutomationPeer) -> str

  

   Gets a name that is used with System.Windows.Automation.Peers.AutomationControlType,to 

    differentiate the control that is represented by this 

    System.Windows.Automation.Peers.AutomationPeer.

  

   Returns: The class name.
  """
  pass
 def GetClassNameCore(self,*args):
  """
  GetClassNameCore(self: AutomationPeer) -> str

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetClassName.

  

   Returns: The class name.
  """
  pass
 def GetClickablePoint(self):
  """
  GetClickablePoint(self: AutomationPeer) -> Point

  

   Gets a System.Windows.Point on the element that is associated with the automation peer that 

    responds to a mouse click.

  

   Returns: A point in the clickable area of the element.
  """
  pass
 def GetClickablePointCore(self,*args):
  """
  GetClickablePointCore(self: AutomationPeer) -> Point

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetClickablePoint.

  

   Returns: A point within the clickable area of the element.
  """
  pass
 def GetHelpText(self):
  """
  GetHelpText(self: AutomationPeer) -> str

  

   Gets text that describes the functionality of the control that is associated with the automation 

    peer.

  

   Returns: The help text.
  """
  pass
 def GetHelpTextCore(self,*args):
  """
  GetHelpTextCore(self: AutomationPeer) -> str

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetHelpText.

  

   Returns: The help text.
  """
  pass
 def GetHostRawElementProviderCore(self,*args):
  """
  GetHostRawElementProviderCore(self: AutomationPeer) -> HostedWindowWrapper

  

   Tells UI Automation where in the UI Automation tree to place the hwnd being hosted by a Windows 

    Presentation Foundation (WPF) element.

  

   Returns: This method returns the hosted hwnd to UI Automation for controls that host hwnd objects.
  """
  pass
 def GetItemStatus(self):
  """
  GetItemStatus(self: AutomationPeer) -> str

  

   Gets text that conveys the visual status of the element that is associated with this automation 

    peer.

  

   Returns: The status.
  """
  pass
 def GetItemStatusCore(self,*args):
  """
  GetItemStatusCore(self: AutomationPeer) -> str

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetItemStatus.

  

   Returns: The status.
  """
  pass
 def GetItemType(self):
  """
  GetItemType(self: AutomationPeer) -> str

  

   Gets a string that describes what kind of item an object represents.

   Returns: The kind of item.
  """
  pass
 def GetItemTypeCore(self,*args):
  """
  GetItemTypeCore(self: AutomationPeer) -> str

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetItemType.

  

   Returns: The kind of item.
  """
  pass
 def GetLabeledBy(self):
  """
  GetLabeledBy(self: AutomationPeer) -> AutomationPeer

  

   Gets the System.Windows.Automation.Peers.AutomationPeer for the System.Windows.Controls.Label 

    that is targeted to the element.

  

   Returns: The System.Windows.Automation.Peers.LabelAutomationPeer for the element that is targeted by the 

    System.Windows.Controls.Label.
  """
  pass
 def GetLabeledByCore(self,*args):
  """
  GetLabeledByCore(self: AutomationPeer) -> AutomationPeer

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetLabeledBy.

  

   Returns: The System.Windows.Automation.Peers.LabelAutomationPeer for the element that is targeted by the 

    System.Windows.Controls.Label.
  """
  pass
 def GetLocalizedControlType(self):
  """
  GetLocalizedControlType(self: AutomationPeer) -> str

  

   Gets a human-readable localized string that represents the 

    System.Windows.Automation.Peers.AutomationControlType value for the control that is associated 

    with this automation peer.

  

   Returns: The type of the control.
  """
  pass
 def GetLocalizedControlTypeCore(self,*args):
  """
  GetLocalizedControlTypeCore(self: AutomationPeer) -> str

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetLocalizedControlType.

  

   Returns: The type of the control.
  """
  pass
 def GetName(self):
  """
  GetName(self: AutomationPeer) -> str

  

   Gets text that describes the element that is associated with this automation peer.

   Returns: The name.
  """
  pass
 def GetNameCore(self,*args):
  """
  GetNameCore(self: AutomationPeer) -> str

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetName.

  

   Returns: The string that contains the label.
  """
  pass
 def GetOrientation(self):
  """
  GetOrientation(self: AutomationPeer) -> AutomationOrientation

  

   Gets a value that indicates the explicit control orientation,if any.

   Returns: The orientation of the control.
  """
  pass
 def GetOrientationCore(self,*args):
  """
  GetOrientationCore(self: AutomationPeer) -> AutomationOrientation

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.GetOrientation.

  

   Returns: The orientation of the control.
  """
  pass
 def GetParent(self):
  """
  GetParent(self: AutomationPeer) -> AutomationPeer

  

   Gets the System.Windows.Automation.Peers.AutomationPeer that is the parent of this 

    System.Windows.Automation.Peers.AutomationPeer.

  

   Returns: The parent automation peer.
  """
  pass
 def GetPattern(self,patternInterface):
  """
  GetPattern(self: AutomationPeer,patternInterface: PatternInterface) -> object

  

   When overridden in a derived class,gets the control pattern that is associated with the 

    specified System.Windows.Automation.Peers.PatternInterface.

  

  

   patternInterface: A value from the System.Windows.Automation.Peers.PatternInterface enumeration.

   Returns: The object that implements the pattern interface; null if this peer does not support this 

    interface.
  """
  pass
 def GetPeerFromPoint(self,point):
  """ GetPeerFromPoint(self: AutomationPeer,point: Point) -> AutomationPeer """
  pass
 def GetPeerFromPointCore(self,*args):
  """ GetPeerFromPointCore(self: AutomationPeer,point: Point) -> AutomationPeer """
  pass
 def HasKeyboardFocus(self):
  """
  HasKeyboardFocus(self: AutomationPeer) -> bool

  

   Gets a value that indicates whether the element that is associated with this automation peer 

    currently has keyboard focus.

  

   Returns: true if the element has keyboard focus; otherwise,false.
  """
  pass
 def HasKeyboardFocusCore(self,*args):
  """
  HasKeyboardFocusCore(self: AutomationPeer) -> bool

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.HasKeyboardFocus.

  

   Returns: true if the element has keyboard focus; otherwise,false.
  """
  pass
 def InvalidatePeer(self):
  """
  InvalidatePeer(self: AutomationPeer)

   Triggers recalculation of the main properties of the 

    System.Windows.Automation.Peers.AutomationPeer and raises the 

    System.ComponentModel.INotifyPropertyChanged.PropertyChanged notification to the Automation 

    Client if the properties changed.
  """
  pass
 def IsContentElement(self):
  """
  IsContentElement(self: AutomationPeer) -> bool

  

   Gets a value that indicates whether the element that is associated with this automation peer 

    contains data that is presented to the user.

  

   Returns: true if the element is a content element; otherwise,false.
  """
  pass
 def IsContentElementCore(self,*args):
  """
  IsContentElementCore(self: AutomationPeer) -> bool

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.IsContentElement.

  

   Returns: true if the element is a content element; otherwise,false.
  """
  pass
 def IsControlElement(self):
  """
  IsControlElement(self: AutomationPeer) -> bool

  

   Gets a value that indicates whether the element is understood by the user as interactive or as 

    contributing to the logical structure of the control in the GUI.

  

   Returns: true if the element is a control; otherwise,false.
  """
  pass
 def IsControlElementCore(self,*args):
  """
  IsControlElementCore(self: AutomationPeer) -> bool

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.IsControlElement.

  

   Returns: true if the element is a control; otherwise,false.
  """
  pass
 def IsEnabled(self):
  """
  IsEnabled(self: AutomationPeer) -> bool

  

   Gets a value that indicates whether the element associated with this automation peer supports 

    interaction.

  

   Returns: true if the element supports interaction; otherwise,false.
  """
  pass
 def IsEnabledCore(self,*args):
  """
  IsEnabledCore(self: AutomationPeer) -> bool

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.IsEnabled.

  

   Returns: true if the automation peer can receive and send events; otherwise,false.
  """
  pass
 def IsKeyboardFocusable(self):
  """
  IsKeyboardFocusable(self: AutomationPeer) -> bool

  

   Gets a value that indicates whether the element can accept keyboard focus.

   Returns: true if the element can accept keyboard focus; otherwise,false.
  """
  pass
 def IsKeyboardFocusableCore(self,*args):
  """
  IsKeyboardFocusableCore(self: AutomationPeer) -> bool

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.IsKeyboardFocusable.

  

   Returns: true if the element can accept keyboard focus; otherwise,false.
  """
  pass
 def IsOffscreen(self):
  """
  IsOffscreen(self: AutomationPeer) -> bool

  

   Gets a value that indicates whether an element is off the screen.

   Returns: true if the element is not on the screen; otherwise,false.
  """
  pass
 def IsOffscreenCore(self,*args):
  """
  IsOffscreenCore(self: AutomationPeer) -> bool

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.IsOffscreen.

  

   Returns: true if the element is not on the screen; otherwise,false.
  """
  pass
 def IsPassword(self):
  """
  IsPassword(self: AutomationPeer) -> bool

  

   Gets a value that indicates whether the element contains sensitive content.

   Returns: true if the element contains sensitive content such as a password; otherwise,false.
  """
  pass
 def IsPasswordCore(self,*args):
  """
  IsPasswordCore(self: AutomationPeer) -> bool

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.IsPassword.

  

   Returns: true if the element contains sensitive content; otherwise,false.
  """
  pass
 def IsRequiredForForm(self):
  """
  IsRequiredForForm(self: AutomationPeer) -> bool

  

   Gets a value that indicates whether the element that is associated with this peer must be 

    completed on a form.

  

   Returns: true if the element must be completed; otherwise,false.
  """
  pass
 def IsRequiredForFormCore(self,*args):
  """
  IsRequiredForFormCore(self: AutomationPeer) -> bool

  

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.IsRequiredForForm.

  

   Returns: true if the element is must be completed; otherwise,false.
  """
  pass
 @staticmethod
 def ListenerExists(eventId):
  """
  ListenerExists(eventId: AutomationEvents) -> bool

  

   Gets a value that indicates whether UI Automation is listening for the specified event.

  

   eventId: One of the enumeration values.

   Returns: A boolean that indicates whether UI Automation is listening for the event.
  """
  pass
 def PeerFromProvider(self,*args):
  """
  PeerFromProvider(self: AutomationPeer,provider: IRawElementProviderSimple) -> AutomationPeer

  

   Gets an System.Windows.Automation.Peers.AutomationPeer for the specified 

    System.Windows.Automation.Provider.IRawElementProviderSimple proxy.

  

  

   provider: The class that implements System.Windows.Automation.Provider.IRawElementProviderSimple.

   Returns: The System.Windows.Automation.Peers.AutomationPeer.
  """
  pass
 def ProviderFromPeer(self,*args):
  """
  ProviderFromPeer(self: AutomationPeer,peer: AutomationPeer) -> IRawElementProviderSimple

  

   Gets the System.Windows.Automation.Provider.IRawElementProviderSimple for the specified 

    System.Windows.Automation.Peers.AutomationPeer.

  

  

   peer: The automation peer.

   Returns: The proxy.
  """
  pass
 def RaiseAsyncContentLoadedEvent(self,args):
  """
  RaiseAsyncContentLoadedEvent(self: AutomationPeer,args: AsyncContentLoadedEventArgs)

   Called by the System.Windows.Automation.Peers.AutomationPeer to raise the 

    System.Windows.Automation.AutomationElement.AsyncContentLoadedEvent event.

  

  

   args: The event data.
  """
  pass
 def RaiseAutomationEvent(self,eventId):
  """
  RaiseAutomationEvent(self: AutomationPeer,eventId: AutomationEvents)

   Raises an automation event.

  

   eventId: The event identifier.
  """
  pass
 def RaisePropertyChangedEvent(self,property,oldValue,newValue):
  """
  RaisePropertyChangedEvent(self: AutomationPeer,property: AutomationProperty,oldValue: object,newValue: object)

   Raises an event to notify the automation client of a changed property value.

  

   property: The property that changed.

   oldValue: The previous value of the property.

   newValue: The new value of the property.
  """
  pass
 def ResetChildrenCache(self):
  """
  ResetChildrenCache(self: AutomationPeer)

   Synchronously resets the tree of child elements by calling 

    System.Windows.Automation.Peers.AutomationPeer.GetChildrenCore.
  """
  pass
 def SetFocus(self):
  """
  SetFocus(self: AutomationPeer)

   Sets the keyboard focus on the element that is associated with this automation peer.
  """
  pass
 def SetFocusCore(self,*args):
  """
  SetFocusCore(self: AutomationPeer)

   When overridden in a derived class,is called by 

    System.Windows.Automation.Peers.AutomationPeer.SetFocus.
  """
  pass
 EventsSource=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets an System.Windows.Automation.Peers.AutomationPeer that is reported to the automation client as a source for all the events that come from this System.Windows.Automation.Peers.AutomationPeer.



Get: EventsSource(self: AutomationPeer) -> AutomationPeer



Set: EventsSource(self: AutomationPeer)=value

"""

 IsHwndHost=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the element that is associated with this System.Windows.Automation.Peers.AutomationPeer hosts hwnds in Windows Presentation Foundation (WPF).



"""



class ContentElementAutomationPeer(AutomationPeer):
 """
 Exposes System.Windows.ContentElement types to UI Automation.

 

 ContentElementAutomationPeer(owner: ContentElement)
 """
 @staticmethod
 def CreatePeerForElement(element):
  """
  CreatePeerForElement(element: ContentElement) -> AutomationPeer

  

   Creates a System.Windows.Automation.Peers.ContentElementAutomationPeer for the specified 

    System.Windows.ContentElement.

  

  

   element: The System.Windows.ContentElement that is associated with this 

    System.Windows.Automation.Peers.ContentElementAutomationPeer.

  

   Returns: The System.Windows.Automation.Peers.ContentElementAutomationPeer for the specified 

    System.Windows.ContentElement.
  """
  pass
 @staticmethod
 def FromElement(element):
  """
  FromElement(element: ContentElement) -> AutomationPeer

  

   Gets the System.Windows.Automation.Peers.ContentElementAutomationPeer for the specified 

    System.Windows.ContentElement.

  

  

   element: The System.Windows.ContentElement that is associated with this 

    System.Windows.Automation.Peers.ContentElementAutomationPeer.

  

   Returns: The System.Windows.Automation.Peers.ContentElementAutomationPeer for the specified 

    System.Windows.ContentElement,or null if the 

    System.Windows.Automation.Peers.ContentElementAutomationPeer has not been created by the 

    System.Windows.Automation.Peers.ContentElementAutomationPeer.CreatePeerForElement(System.Windows.

    ContentElement) method.
  """
  pass
 def GetPattern(self,patternInterface):
  """
  GetPattern(self: ContentElementAutomationPeer,patternInterface: PatternInterface) -> object

  

   Gets the control pattern for the System.Windows.ContentElement that is associated with this 

    System.Windows.Automation.Peers.ContentElementAutomationPeer.

  

  

   patternInterface: One of the enumeration values.

   Returns: An object that implements the System.Windows.Automation.Provider.ISynchronizedInputProvider 

    interface if patternInterface is 

    System.Windows.Automation.Peers.PatternInterface.SynchronizedInput; otherwise,null.
  """
  pass
 @staticmethod
 def __new__(self,owner):
  """ __new__(cls: type,owner: ContentElement) """
  pass
 IsHwndHost=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the element that is associated with this System.Windows.Automation.Peers.AutomationPeer hosts hwnds in Windows Presentation Foundation (WPF).



"""

 Owner=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.ContentElement that is associated with this System.Windows.Automation.Peers.ContentElementAutomationPeer.



Get: Owner(self: ContentElementAutomationPeer) -> ContentElement



"""



class UIElementAutomationPeer(AutomationPeer):
 """
 Exposes System.Windows.UIElement types to UI Automation.

 

 UIElementAutomationPeer(owner: UIElement)
 """
 @staticmethod
 def CreatePeerForElement(element):
  """
  CreatePeerForElement(element: UIElement) -> AutomationPeer

  

   Creates a System.Windows.Automation.Peers.UIElementAutomationPeer for the specified 

    System.Windows.UIElement.

  

  

   element: The System.Windows.UIElement that is associated with this 

    System.Windows.Automation.Peers.UIElementAutomationPeer.

  

   Returns: A System.Windows.Automation.Peers.UIElementAutomationPeer.
  """
  pass
 @staticmethod
 def FromElement(element):
  """
  FromElement(element: UIElement) -> AutomationPeer

  

   Gets the System.Windows.Automation.Peers.UIElementAutomationPeer for the specified 

    System.Windows.UIElement.

  

  

   element: The System.Windows.UIElement that is associated with this 

    System.Windows.Automation.Peers.UIElementAutomationPeer.

  

   Returns: The System.Windows.Automation.Peers.UIElementAutomationPeer; or null,if the 

    System.Windows.Automation.Peers.UIElementAutomationPeer was not created by the 

    System.Windows.Automation.Peers.UIElementAutomationPeer.CreatePeerForElement(System.Windows.UIEle

    ment) method.
  """
  pass
 def GetPattern(self,patternInterface):
  """
  GetPattern(self: UIElementAutomationPeer,patternInterface: PatternInterface) -> object

  

   Gets the control pattern for the System.Windows.UIElement that is associated with this 

    System.Windows.Automation.Peers.UIElementAutomationPeer.

  

  

   patternInterface: A value from the enumeration.

   Returns: An object that implements the System.Windows.Automation.Provider.ISynchronizedInputProvider 

    interface if patternInterface is 

    System.Windows.Automation.Peers.PatternInterface.SynchronizedInput; otherwise,null.
  """
  pass
 @staticmethod
 def __new__(self,owner):
  """ __new__(cls: type,owner: UIElement) """
  pass
 IsHwndHost=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the element that is associated with this System.Windows.Automation.Peers.AutomationPeer hosts hwnds in Windows Presentation Foundation (WPF).



"""

 Owner=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.UIElement that is associated with this System.Windows.Automation.Peers.UIElementAutomationPeer.



Get: Owner(self: UIElementAutomationPeer) -> UIElement



"""



class GenericRootAutomationPeer(UIElementAutomationPeer):
 """
 Exposes System.Windows.Interop.HwndSource types to UI Automation.

 

 GenericRootAutomationPeer(owner: UIElement)
 """
 @staticmethod
 def __new__(self,owner):
  """ __new__(cls: type,owner: UIElement) """
  pass
 IsHwndHost=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the element that is associated with this System.Windows.Automation.Peers.AutomationPeer hosts hwnds in Windows Presentation Foundation (WPF).



"""



class HostedWindowWrapper(object):
 """
 Exposes System.Windows.Interop.HwndHost types to UI Automation.

 

 HostedWindowWrapper(hwnd: IntPtr)
 """
 @staticmethod
 def __new__(self,hwnd):
  """ __new__(cls: type,hwnd: IntPtr) """
  pass

class PatternInterface(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the control pattern that System.Windows.Automation.Peers.AutomationPeer.GetPattern(System.Windows.Automation.Peers.PatternInterface) returns.

 

 enum PatternInterface,values: Dock (12),ExpandCollapse (6),Grid (7),GridItem (8),Invoke (0),ItemContainer (18),MultipleView (9),RangeValue (3),Scroll (4),ScrollItem (5),Selection (1),SelectionItem (11),SynchronizedInput (20),Table (13),TableItem (14),Text (17),Toggle (15),Transform (16),Value (2),VirtualizedItem (19),Window (10)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Dock=None
 ExpandCollapse=None
 Grid=None
 GridItem=None
 Invoke=None
 ItemContainer=None
 MultipleView=None
 RangeValue=None
 Scroll=None
 ScrollItem=None
 Selection=None
 SelectionItem=None
 SynchronizedInput=None
 Table=None
 TableItem=None
 Text=None
 Toggle=None
 Transform=None
 Value=None
 value__=None
 VirtualizedItem=None
 Window=None


class UIElement3DAutomationPeer(AutomationPeer):
 """
 Exposes System.Windows.UIElement3D types to UI Automation.

 

 UIElement3DAutomationPeer(owner: UIElement3D)
 """
 @staticmethod
 def CreatePeerForElement(element):
  """
  CreatePeerForElement(element: UIElement3D) -> AutomationPeer

  

   Creates a System.Windows.Automation.Peers.UIElement3DAutomationPeer for the specified 

    System.Windows.UIElement3D.

  

  

   element: The System.Windows.UIElement3D that is associated with this 

    System.Windows.Automation.Peers.UIElement3DAutomationPeer.

  

   Returns: A  System.Windows.Automation.Peers.UIElement3DAutomationPeer for the specified 

    System.Windows.UIElement3D.
  """
  pass
 @staticmethod
 def FromElement(element):
  """
  FromElement(element: UIElement3D) -> AutomationPeer

  

   Returns the System.Windows.Automation.Peers.UIElement3DAutomationPeer for the specified 

    System.Windows.UIElement3D.

  

  

   element: The System.Windows.UIElement3D that is associated with this 

    System.Windows.Automation.Peers.UIElement3DAutomationPeer.

  

   Returns: The System.Windows.Automation.Peers.UIElement3DAutomationPeer,or null if the 

    System.Windows.Automation.Peers.UIElement3DAutomationPeer was not created by the 

    System.Windows.Automation.Peers.UIElement3DAutomationPeer.CreatePeerForElement(System.Windows.UIE

    lement3D) method.
  """
  pass
 def GetPattern(self,patternInterface):
  """
  GetPattern(self: UIElement3DAutomationPeer,patternInterface: PatternInterface) -> object

  

   Returns the control pattern for the System.Windows.UIElement3D that is associated with this 

    System.Windows.Automation.Peers.UIElement3DAutomationPeer.

  

  

   patternInterface: One of the enumeration values.

   Returns: An object that implements the System.Windows.Automation.Provider.ISynchronizedInputProvider 

    interface if patternInterface is 

    System.Windows.Automation.Peers.PatternInterface.SynchronizedInput; otherwise,null.
  """
  pass
 @staticmethod
 def __new__(self,owner):
  """ __new__(cls: type,owner: UIElement3D) """
  pass
 IsHwndHost=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the element that is associated with this System.Windows.Automation.Peers.AutomationPeer hosts hwnds in Windows Presentation Foundation (WPF).



"""

 Owner=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.UIElement3D that is associated with this System.Windows.Automation.Peers.UIElement3DAutomationPeer.



Get: Owner(self: UIElement3DAutomationPeer) -> UIElement3D



"""






class DpiChangedEventArgs(CancelEventArgs):
 # no doc
 def ToString(self):
  """ ToString(self: DpiChangedEventArgs) -> str """
  pass
 DeviceDpiNew=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: DeviceDpiNew(self: DpiChangedEventArgs) -> int



"""

 DeviceDpiOld=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: DeviceDpiOld(self: DpiChangedEventArgs) -> int



"""

 SuggestedRectangle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: SuggestedRectangle(self: DpiChangedEventArgs) -> Rectangle



"""





class PropertyChangedEventArgs(EventArgs):
 """
 Provides data for the System.ComponentModel.INotifyPropertyChanged.PropertyChanged event.

 

 PropertyChangedEventArgs(propertyName: str)
 """
 @staticmethod
 def __new__(self,propertyName):
  """ __new__(cls: type,propertyName: str) """
  pass
 PropertyName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the property that changed.



Get: PropertyName(self: PropertyChangedEventArgs) -> str



"""





# -*- coding: utf-8 -*-
"""Provide conversion services between python.locale and host languages."""
# https://www.science.co.il/language/Locale-codes.php
from pyrevit import HOST_APP
from pyrevit.api import ApplicationServices
from pyrevit.userconfig import user_config


DEFAULT_LANG_DIR = 'LTR'


class AppLocale(object):
    """Type representing a language option."""
    def __init__(self,
                 lang_type,
                 locale_codes,
                 lang_name=None,
                 lang_dir=DEFAULT_LANG_DIR):
        if isinstance(lang_type, ApplicationServices.LanguageType):
            self.lang_type = lang_type
        elif isinstance(lang_type, str):
            self.lang_type = lang_type
        self.lang_name = lang_name
        self.lang_dir = lang_dir
        self.locale_codes = locale_codes
        if self.locale_codes:
            self.locale_code = self.locale_codes[0]

    def __str__(self):
        if self.lang_name:
            return '%s / %s (%s)' % (
                self.lang_name, str(self.lang_type), self.locale_code)
        else:
            return '%s (%s)' % (str(self.lang_type), self.locale_code)

    def __repr__(self):
        return str(self)


DEFAULT_LOCALE = AppLocale(
    lang_type=ApplicationServices.LanguageType.English_USA,
    lang_name="English USA",
    locale_codes=["en_us", "english"])

APP_LOCALES = [
    DEFAULT_LOCALE,

    AppLocale(
        lang_type=ApplicationServices.LanguageType.German,
        lang_name="Deutsch",
        locale_codes=["de_de", "german"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Spanish,
        lang_name="español",
        locale_codes=["es_es", "spanish"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.French,
        lang_name="français",
        locale_codes=["fr_fr", "french"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Italian,
        lang_name="italiano",
        locale_codes=["it_it", "italian"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Dutch,
        lang_name="Nederlands",
        locale_codes=["nl_nl", "nl_be", "dutch"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Chinese_Simplified,
        lang_name="简体中文",
        locale_codes=["chinese_s", "chinese"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Chinese_Traditional,
        lang_name="繁體中文",
        locale_codes=["chinese_t", "chinese"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Japanese,
        lang_name="日本語",
        locale_codes=["ja", "japanese"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Korean,
        lang_name="한국어",
        locale_codes=["ko", "korean"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Russian,
        lang_name="Русский",
        locale_codes=["ru", "russian"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Czech,
        lang_name="Čeština",
        locale_codes=["cs", "czech"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Polish,
        lang_name="Polski",
        locale_codes=["pl", "polish"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Hungarian,
        lang_name="Magyar",
        locale_codes=["hu", "hungarian"]),
    AppLocale(
        lang_type=ApplicationServices.LanguageType.Brazilian_Portuguese,
        lang_name="Português do Brasil",
        locale_codes=["pt_br", "portuguese_brazil", "brazilian",
                    "portuguese", "pt_pt"]),
]

# add version specific languages
if HOST_APP.is_newer_than(2018, or_equal=True):
    APP_LOCALES.append(
        AppLocale(
            lang_type=ApplicationServices.LanguageType.English_GB,
            lang_name="English Great Britain",
            locale_codes=["en_gb"])
        )

# add custom languages provided by this module
APP_LOCALES.append(
    AppLocale(
        lang_type="Bulgarian",
        lang_name="Български",
        locale_codes=["bg", "bulgarian"])
    )
APP_LOCALES.append(
    AppLocale(
        lang_type="Farsi",
        locale_codes=["fa", "farsi", "persian"],
        lang_name="فارسی",
        lang_dir='RTL'
    ))
APP_LOCALES.append(
    AppLocale(
        lang_type="Arabic",
        locale_codes=["ar", "arabic"],
        lang_name="العربیه",
        lang_dir='RTL'
    ))
APP_LOCALES.append(
    AppLocale(
        lang_type="Ukrainian",
        locale_codes=["uk", "ukrainian"],
        lang_name="Українська"
    ))



def get_applocale_by_local_code(locale_code):
    """Return application locale by locale code.

    Args:
        locale_code (str): locale code

    Returns:
        (AppLocale): application locale
    """
    for applocale in APP_LOCALES:
        if locale_code in applocale.locale_codes:
            return applocale


def get_applocale_by_lang_type(lang_type):
    """Return application locale by language type.

    Args:
        lang_type (ApplicationServices.LanguageType | str): language type

    Returns:
        (AppLocale): application locale
    """
    for applocale in APP_LOCALES:
        if lang_type == applocale.lang_type:
            return applocale


def get_applocale_by_lang_name(lang_name):
    """Return application locale by language name.

    Args:
        lang_name (str): language name

    Returns:
        (AppLocale): application locale
    """
    for applocale in APP_LOCALES:
        if lang_name in {applocale.lang_name, str(applocale.lang_type)}:
            return applocale


def get_current_applocale():
    """Return the current locale.
    
    This is the user locale, if set, or the host application locale otherwise.

    Returns:
        (AppLocale): current locale
    """
    if user_config.user_locale:
        return get_applocale_by_local_code(user_config.user_locale)
    return get_applocale_by_lang_type(HOST_APP.language)


def get_host_applocale():
    """Return host application locale.

    Returns:
        (AppLocale): host application locale
    """
    return get_applocale_by_lang_type(HOST_APP.language)


def get_locale_string(string_dict):
    """Returns the correct string from given dict based on host language.

    Args:
        string_dict (dict[str, str]): dict of strings in various locales

    Returns:
        (str): string in correct locale

    Examples:
        ```python
        data = {"en_us":"Hello", "chinese_s":"你好"}
        from pyrevit.coreutils import applocales
        # assuming running Revit is Chinese
        applocales.get_locale_string(data)
        ```
        "你好"
    """
    applocale = get_applocale_by_local_code(user_config.user_locale)
    if applocale:
        local_codes = applocale.locale_codes + DEFAULT_LOCALE.locale_codes
    else:
        local_codes = DEFAULT_LOCALE.locale_codes
    for locale_code in local_codes:
        if locale_code in string_dict:
            return string_dict[locale_code]



class RetrieveVirtualItemEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.ListView.RetrieveVirtualItem event.

 

 RetrieveVirtualItemEventArgs(itemIndex: int)
 """
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 @staticmethod
 def __new__(self,itemIndex):
  """ __new__(cls: type,itemIndex: int) """
  pass
 def __setitem__(self,*args):
  """ x.__setitem__(i,y) <==> x[i]= """
  pass
 Item=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the item retrieved from the cache.



Get: Item(self: RetrieveVirtualItemEventArgs) -> ListViewItem



Set: Item(self: RetrieveVirtualItemEventArgs)=value

"""

 ItemIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of the item to retrieve from the cache.



Get: ItemIndex(self: RetrieveVirtualItemEventArgs) -> int



"""





class ToolboxBitmapAttribute(Attribute,_Attribute):
 """
 Allows you to specify an icon to represent a control in a container,such as the Microsoft Visual Studio Form Designer.

 

 ToolboxBitmapAttribute(imageFile: str)

 ToolboxBitmapAttribute(t: Type)

 ToolboxBitmapAttribute(t: Type,name: str)
 """
 def Equals(self,value):
  """
  Equals(self: ToolboxBitmapAttribute,value: object) -> bool

  

   Indicates whether the specified object is a System.Drawing.ToolboxBitmapAttribute object and is 

    identical to this System.Drawing.ToolboxBitmapAttribute object.

  

  

   value: The System.Object to test.

   Returns: This method returns true if value is both a System.Drawing.ToolboxBitmapAttribute object and is 

    identical to this System.Drawing.ToolboxBitmapAttribute object.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: ToolboxBitmapAttribute) -> int

  

   Gets a hash code for this System.Drawing.ToolboxBitmapAttribute object.

   Returns: The hash code for this System.Drawing.ToolboxBitmapAttribute object.
  """
  pass
 def GetImage(self,*__args):
  """
  GetImage(self: ToolboxBitmapAttribute,type: Type,large: bool) -> Image

  

   Gets the small or large System.Drawing.Image associated with this 

    System.Drawing.ToolboxBitmapAttribute object.

  

  

   type: If this System.Drawing.ToolboxBitmapAttribute object does not already have a small image,this 

    method searches for a bitmap resource in the assembly that defines the type specified by the 

    component type. For example,if you pass typeof(ControlA) to the type parameter,then this 

    method searches the assembly that defines ControlA.

  

   large: Specifies whether this method returns a large image (true) or a small image (false). The small 

    image is 16 by 16,and the large image is 32 by 32.

  

   Returns: An System.Drawing.Image associated with this System.Drawing.ToolboxBitmapAttribute object.

  GetImage(self: ToolboxBitmapAttribute,type: Type,imgName: str,large: bool) -> Image

  

   Gets the small or large System.Drawing.Image associated with this 

    System.Drawing.ToolboxBitmapAttribute object.

  

  

   type: If this System.Drawing.ToolboxBitmapAttribute object does not already have a small image,this 

    method searches for an embedded bitmap resource in the assembly that defines the type specified 

    by the component type. For example,if you pass typeof(ControlA) to the type parameter,then 

    this method searches the assembly that defines ControlA.

  

   imgName: The name of the embedded bitmap resource.

   large: Specifies whether this method returns a large image (true) or a small image (false). The small 

    image is 16 by 16,and the large image is 32 by 32.

  

   Returns: An System.Drawing.Image associated with this System.Drawing.ToolboxBitmapAttribute object.

  GetImage(self: ToolboxBitmapAttribute,type: Type) -> Image

  

   Gets the small System.Drawing.Image associated with this System.Drawing.ToolboxBitmapAttribute 

    object.

  

  

   type: If this System.Drawing.ToolboxBitmapAttribute object does not already have a small image,this 

    method searches for a bitmap resource in the assembly that defines the type specified by the 

    type parameter. For example,if you pass typeof(ControlA) to the type parameter,then this 

    method searches the assembly that defines ControlA.

  

   Returns: The small System.Drawing.Image associated with this System.Drawing.ToolboxBitmapAttribute object.

  GetImage(self: ToolboxBitmapAttribute,component: object) -> Image

  

   Gets the small System.Drawing.Image associated with this System.Drawing.ToolboxBitmapAttribute 

    object.

  

  

   component: If this System.Drawing.ToolboxBitmapAttribute object does not already have a small image,this 

    method searches for a bitmap resource in the assembly that defines the type of the object 

    specified by the component parameter. For example,if you pass an object of type ControlA to the 

    component parameter,then this method searches the assembly that defines ControlA.

  

   Returns: The small System.Drawing.Image associated with this System.Drawing.ToolboxBitmapAttribute object.

  GetImage(self: ToolboxBitmapAttribute,component: object,large: bool) -> Image

  

   Gets the small or large System.Drawing.Image associated with this 

    System.Drawing.ToolboxBitmapAttribute object.

  

  

   component: If this System.Drawing.ToolboxBitmapAttribute object does not already have a small image,this 

    method searches for a bitmap resource in the assembly that defines the type of the object 

    specified by the component parameter. For example,if you pass an object of type ControlA to the 

    component parameter,then this method searches the assembly that defines ControlA.

  

   large: Specifies whether this method returns a large image (true) or a small image (false). The small 

    image is 16 by 16,and the large image is 32 by 32.

  

   Returns: An System.Drawing.Image object associated with this System.Drawing.ToolboxBitmapAttribute object.
  """
  pass
 @staticmethod
 def GetImageFromResource(t,imageName,large):
  """
  GetImageFromResource(t: Type,imageName: str,large: bool) -> Image

  

   Returns an System.Drawing.Image object based on a bitmap resource that is embedded in an 

    assembly.

  

  

   t: This method searches for an embedded bitmap resource in the assembly that defines the type 

    specified by the t parameter. For example,if you pass typeof(ControlA) to the t parameter,then 

    this method searches the assembly that defines ControlA.

  

   imageName: The name of the embedded bitmap resource.

   large: Specifies whether this method returns a large image (true)or a small image (false). The small 

    image is 16 by 16,and the large image is 32 x 32.

  

   Returns: An System.Drawing.Image object based on the retrieved bitmap.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,imageFile: str)

  __new__(cls: type,t: Type)

  __new__(cls: type,t: Type,name: str)
  """
  pass
 def __ne__(self,*args):
  pass
 Default=None


# variables with complex values




class DiscreteVectorKeyFrame(VectorKeyFrame,ISealable,IKeyFrame):
 """
 Animates from the System.Windows.Vector value of the previous key frame to its own System.Windows.Media.Animation.VectorKeyFrame.Value using discrete interpolation.

 

 DiscreteVectorKeyFrame()

 DiscreteVectorKeyFrame(value: Vector)

 DiscreteVectorKeyFrame(value: Vector,keyTime: KeyTime)
 """
 def CloneCore(self,*args):
  """
  CloneCore(self: Freezable,sourceFreezable: Freezable)

   Makes the instance a clone (deep copy) of the specified System.Windows.Freezable using base 

    (non-animated) property values.

  

  

   sourceFreezable: The object to clone.
  """
  pass
 def CloneCurrentValueCore(self,*args):
  """
  CloneCurrentValueCore(self: Freezable,sourceFreezable: Freezable)

   Makes the instance a modifiable clone (deep copy) of the specified System.Windows.Freezable 

    using current property values.

  

  

   sourceFreezable: The System.Windows.Freezable to be cloned.
  """
  pass
 def CreateInstance(self,*args):
  """
  CreateInstance(self: Freezable) -> Freezable

  

   Initializes a new instance of the System.Windows.Freezable class.

   Returns: The new instance.
  """
  pass
 def CreateInstanceCore(self,*args):
  """
  CreateInstanceCore(self: DiscreteVectorKeyFrame) -> Freezable

  

   Creates a new instance of System.Windows.Media.Animation.DiscreteVectorKeyFrame.

   Returns: A new System.Windows.Media.Animation.DiscreteVectorKeyFrame.
  """
  pass
 def FreezeCore(self,*args):
  """
  FreezeCore(self: Freezable,isChecking: bool) -> bool

  

   Makes the System.Windows.Freezable object unmodifiable or tests whether it can be made 

    unmodifiable.

  

  

   isChecking: true to return an indication of whether the object can be frozen (without actually freezing it); 

    false to actually freeze the object.

  

   Returns: If isChecking is true,this method returns true if the System.Windows.Freezable can be made 

    unmodifiable,or false if it cannot be made unmodifiable. If isChecking is false,this method 

    returns true if the if the specified System.Windows.Freezable is now unmodifiable,or false if 

    it cannot be made unmodifiable.
  """
  pass
 def GetAsFrozenCore(self,*args):
  """
  GetAsFrozenCore(self: Freezable,sourceFreezable: Freezable)

   Makes the instance a frozen clone of the specified System.Windows.Freezable using base 

    (non-animated) property values.

  

  

   sourceFreezable: The instance to copy.
  """
  pass
 def GetCurrentValueAsFrozenCore(self,*args):
  """
  GetCurrentValueAsFrozenCore(self: Freezable,sourceFreezable: Freezable)

   Makes the current instance a frozen clone of the specified System.Windows.Freezable. If the 

    object has animated dependency properties,their current animated values are copied.

  

  

   sourceFreezable: The System.Windows.Freezable to copy and freeze.
  """
  pass
 def InterpolateValueCore(self,*args):
  """
  InterpolateValueCore(self: DiscreteVectorKeyFrame,baseValue: Vector,keyFrameProgress: float) -> Vector

  

   Uses discrete interpolation to transition between the previous key frame value and the value of 

    the current key frame.

  

  

   baseValue: The value to animate from.

   keyFrameProgress: A value between 0.0 and 1.0,inclusive,that specifies the percentage of time that has elapsed 

    for this key frame.

  

   Returns: The output value of this key frame given the specified base value and progress.
  """
  pass
 def OnChanged(self,*args):
  """
  OnChanged(self: Freezable)

   Called when the current System.Windows.Freezable object is modified.
  """
  pass
 def OnFreezablePropertyChanged(self,*args):
  """
  OnFreezablePropertyChanged(self: Freezable,oldValue: DependencyObject,newValue: DependencyObject,property: DependencyProperty)

   This member supports the Windows Presentation Foundation (WPF) infrastructure and is not 

    intended to be used directly from your code.

  

  

   oldValue: The previous value of the data member.

   newValue: The current value of the data member.

   property: The property that changed.

  OnFreezablePropertyChanged(self: Freezable,oldValue: DependencyObject,newValue: DependencyObject)

   Ensures that appropriate context pointers are established for a 

    System.Windows.DependencyObjectType data member that has just been set.

  

  

   oldValue: The previous value of the data member.

   newValue: The current value of the data member.
  """
  pass
 def OnPropertyChanged(self,*args):
  """
  OnPropertyChanged(self: Freezable,e: DependencyPropertyChangedEventArgs)

   Overrides the System.Windows.DependencyObject implementation of 

    System.Windows.DependencyObject.OnPropertyChanged(System.Windows.DependencyPropertyChangedEventAr

    gs) to also invoke any System.Windows.Freezable.Changed handlers in response to a changing 

    dependency property of type System.Windows.Freezable.

  

  

   e: Event data that contains information about which property changed,and its old and new values.
  """
  pass
 def ReadPreamble(self,*args):
  """
  ReadPreamble(self: Freezable)

   Ensures that the System.Windows.Freezable is being accessed from a valid thread. Inheritors of 

    System.Windows.Freezable must call this method at the beginning of any API that reads data 

    members that are not dependency properties.
  """
  pass
 def ShouldSerializeProperty(self,*args):
  """
  ShouldSerializeProperty(self: DependencyObject,dp: DependencyProperty) -> bool

  

   Returns a value that indicates whether serialization processes should serialize the value for 

    the provided dependency property.

  

  

   dp: The identifier for the dependency property that should be serialized.

   Returns: true if the dependency property that is supplied should be value-serialized; otherwise,false.
  """
  pass
 def WritePostscript(self,*args):
  """
  WritePostscript(self: Freezable)

   Raises the System.Windows.Freezable.Changed event for the System.Windows.Freezable and invokes 

    its System.Windows.Freezable.OnChanged method. Classes that derive from System.Windows.Freezable 

    should call this method at the end of any API that modifies class members that are not stored as 

    dependency properties.
  """
  pass
 def WritePreamble(self,*args):
  """
  WritePreamble(self: Freezable)

   Verifies that the System.Windows.Freezable is not frozen and that it is being accessed from a 

    valid threading context. System.Windows.Freezable inheritors should call this method at the 

    beginning of any API that writes to data members that are not dependency properties.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,value=None,keyTime=None):
  """
  __new__(cls: type)

  __new__(cls: type,value: Vector)

  __new__(cls: type,value: Vector,keyTime: KeyTime)
  """
  pass



class ItemCheckEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.CheckedListBox.ItemCheck event of the System.Windows.Forms.CheckedListBox and System.Windows.Forms.ListView controls.

 

 ItemCheckEventArgs(index: int,newCheckValue: CheckState,currentValue: CheckState)
 """
 @staticmethod
 def __new__(self,index,newCheckValue,currentValue):
  """ __new__(cls: type,index: int,newCheckValue: CheckState,currentValue: CheckState) """
  pass
 CurrentValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating the current state of the item's check box.



Get: CurrentValue(self: ItemCheckEventArgs) -> CheckState



"""

 Index=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the zero-based index of the item to change.



Get: Index(self: ItemCheckEventArgs) -> int



"""

 NewValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether to set the check box for the item to be checked,unchecked,or indeterminate.



Get: NewValue(self: ItemCheckEventArgs) -> CheckState



Set: NewValue(self: ItemCheckEventArgs)=value

"""





class DataGridColumnReorderingEventArgs(DataGridColumnEventArgs):
 """
 Provides data for the System.Windows.Controls.DataGrid.ColumnReordering event.

 

 DataGridColumnReorderingEventArgs(dataGridColumn: DataGridColumn)
 """
 @staticmethod
 def __new__(self,dataGridColumn):
  """ __new__(cls: type,dataGridColumn: DataGridColumn) """
  pass
 Cancel=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether the reordering operation is stopped before completion.



Get: Cancel(self: DataGridColumnReorderingEventArgs) -> bool



Set: Cancel(self: DataGridColumnReorderingEventArgs)=value

"""

 DragIndicator=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the control that is used to display the visual indicator of the header for the column that is being dragged.



Get: DragIndicator(self: DataGridColumnReorderingEventArgs) -> Control



Set: DragIndicator(self: DataGridColumnReorderingEventArgs)=value

"""

 DropLocationIndicator=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the control that is used to display the visual indicator of the current drop location during a column drag operation.



Get: DropLocationIndicator(self: DataGridColumnReorderingEventArgs) -> Control



Set: DropLocationIndicator(self: DataGridColumnReorderingEventArgs)=value

"""





class String(object):
 """
 Represents text as a series of Unicode characters.

 

 str(value: Char*)

 str(value: Char*,startIndex: int,length: int)

 str(value: SByte*)

 str(value: SByte*,startIndex: int,length: int)

 str(value: SByte*,startIndex: int,length: int,enc: Encoding)

 str(value: Array[Char],startIndex: int,length: int)

 str(value: Array[Char])

 str(c: Char,count: int)
 """
 def capitalize(self,*args):
  """
  capitalize(self: str) -> str

  

   Returns a copy of this string converted to uppercase
  """
  pass
 def center(self,*args):
  """
  center(self: str,width: int,fillchar: Char) -> str

  center(self: str,width: int) -> str
  """
  pass
 def count(self,*args):
  """
  count(self: str,ssub: str,start: int,end: int) -> int

  count(self: str,sub: str,start: int) -> int

  count(self: str,sub: str) -> int
  """
  pass
 def decode(self,*args):
  """
  decode(s: str,encoding: object,errors: str) -> str

  decode(s: str) -> str
  """
  pass
 def encode(self,*args):
  """ encode(s: str,encoding: object,errors: str) -> str """
  pass
 def endswith(self,*args):
  """
  endswith(self: str,suffix: object) -> bool

  endswith(self: str,suffix: object,start: int) -> bool

  endswith(self: str,suffix: object,start: int,end: int) -> bool

  endswith(self: str,suffix: str) -> bool

  endswith(self: str,suffix: str,start: int) -> bool

  endswith(self: str,suffix: str,start: int,end: int) -> bool
  """
  pass
 def expandtabs(self,*args):
  """
  expandtabs(self: str,tabsize: int) -> str

  expandtabs(self: str) -> str
  """
  pass
 def find(self,*args):
  """
  find(self: str,sub: str,start: int,end: int) -> int

  find(self: str,sub: str,start: long,end: long) -> int

  find(self: str,sub: str,start: object,end: object) -> int

  find(self: str,sub: str) -> int

  find(self: str,sub: str,start: int) -> int

  find(self: str,sub: str,start: long) -> int
  """
  pass
 def format(self,*args):
  """
  format(format_string�: str,**kwargs�: dict,*args�: Array[object]) -> str

  format(format_string: str,*args: Array[object]) -> str

  

   Replaces each replacement field in the string with the provided arguments.

     

      

      replacement_field= "{" field_name ["!" conversion] [":" format_spec] "}"

     

    field_name = (identifier | integer) ("." identifier | "[" element_index "]")*

      

      

     format_spec: [[fill]align][sign][#][0][width][,][.precision][type]

     


    
     Conversion can be 'r' for repr or 's' for string.
  """
  pass
 def index(self,*args):
  """
  index(self: str,sub: str,start: int,end: int) -> int

  index(self: str,sub: str,start: object,end: object) -> int

  index(self: str,sub: str) -> int

  index(self: str,sub: str,start: int) -> int
  """
  pass
 def isalnum(self,*args):
  """ isalnum(self: str) -> bool """
  pass
 def isalpha(self,*args):
  """ isalpha(self: str) -> bool """
  pass
 def isdecimal(self,*args):
  """ isdecimal(self: str) -> bool """
  pass
 def isdigit(self,*args):
  """ isdigit(self: str) -> bool """
  pass
 def islower(self,*args):
  """ islower(self: str) -> bool """
  pass
 def isnumeric(self,*args):
  """ isnumeric(self: str) -> bool """
  pass
 def isspace(self,*args):
  """ isspace(self: str) -> bool """
  pass
 def istitle(self,*args):
  """
  istitle(self: str) -> bool

  

   return true if self is a titlecased string and there is at least one

     character in 

    self; also,uppercase characters may only follow uncased

     characters (e.g. 

    whitespace) and lowercase characters only cased ones.

     return false otherwise.
  """
  pass
 def isunicode(self,*args):
  """ isunicode(self: str) -> bool """
  pass
 def isupper(self,*args):
  """ isupper(self: str) -> bool """
  pass
 def join(self,*args):
  """
  join(self: str,sequence: list) -> str

  join(self: str,sequence: object) -> str

  

   Return a string which is the concatenation of the strings 

     in the sequence seq. The 

    separator between elements is the 

     string providing this method
  """
  pass
 def ljust(self,*args):
  """
  ljust(self: str,width: int,fillchar: Char) -> str

  ljust(self: str,width: int) -> str
  """
  pass
 def lower(self,*args):
  """ lower(self: str) -> str """
  pass
 def lstrip(self,*args):
  """
  lstrip(self: str,chars: str) -> str

  lstrip(self: str) -> str
  """
  pass
 def partition(self,*args):
  """ partition(self: str,sep: str) -> tuple (of str) """
  pass
 def replace(self,*args):
  """ replace(self: str,old: str,new: str,count: int) -> str """
  pass
 def rfind(self,*args):
  """
  rfind(self: str,sub: str,start: int,end: int) -> int

  rfind(self: str,sub: str,start: long,end: long) -> int

  rfind(self: str,sub: str,start: object,end: object) -> int

  rfind(self: str,sub: str) -> int

  rfind(self: str,sub: str,start: int) -> int

  rfind(self: str,sub: str,start: long) -> int
  """
  pass
 def rindex(self,*args):
  """
  rindex(self: str,sub: str,start: int,end: int) -> int

  rindex(self: str,sub: str,start: object,end: object) -> int

  rindex(self: str,sub: str) -> int

  rindex(self: str,sub: str,start: int) -> int
  """
  pass
 def rjust(self,*args):
  """
  rjust(self: str,width: int,fillchar: Char) -> str

  rjust(self: str,width: int) -> str
  """
  pass
 def rpartition(self,*args):
  """ rpartition(self: str,sep: str) -> tuple (of str) """
  pass
 def rsplit(self,*args):
  """
  rsplit(self: str) -> list

  rsplit(self: str,sep: str,maxsplit: int) -> list

  rsplit(self: str,sep: str) -> list
  """
  pass
 def rstrip(self,*args):
  """
  rstrip(self: str,chars: str) -> str

  rstrip(self: str) -> str
  """
  pass
 def split(self,*args):
  """
  split(self: str,sep: str,maxsplit: int) -> list

  split(self: str,sep: str) -> list

  split(self: str) -> list
  """
  pass
 def splitlines(self,*args):
  """
  splitlines(self: str,keepends: bool) -> list

  splitlines(self: str) -> list
  """
  pass
 def startswith(self,*args):
  """
  startswith(self: str,prefix: str) -> bool

  startswith(self: str,prefix: str,start: int) -> bool

  startswith(self: str,prefix: str,start: int,end: int) -> bool

  startswith(self: str,prefix: object) -> bool

  startswith(self: str,prefix: object,start: int) -> bool

  startswith(self: str,prefix: object,start: int,end: int) -> bool
  """
  pass
 def strip(self,*args):
  """
  strip(self: str,chars: str) -> str

  strip(self: str) -> str
  """
  pass
 def swapcase(self,*args):
  """ swapcase(self: str) -> str """
  pass
 def title(self,*args):
  """ title(self: str) -> str """
  pass
 def translate(self,*args):
  """
  translate(self: str,table: str,deletechars: str) -> str

  translate(self: str,table: str) -> str

  translate(self: str,table: dict) -> str
  """
  pass
 def upper(self,*args):
  """ upper(self: str) -> str """
  pass
 def zfill(self,*args):
  """ zfill(self: str,width: int) -> str """
  pass
 def _formatter_field_name_split(self,*args):
  """ _formatter_field_name_split(self: str) -> tuple """
  pass
 def _formatter_parser(self,*args):
  """ _formatter_parser(self: str) -> IEnumerable[tuple] """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+yx.__add__(y) <==> x+y """
  pass
 def __contains__(self,*args):
  """
  __contains__(s: str,item: Char) -> bool

  __contains__(s: str,item: str) -> bool
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y]x.__getitem__(y) <==> x[y]x.__getitem__(y) <==> x[y] """
  pass
 def __getnewargs__(self,*args):
  """ __getnewargs__(self: str) -> object """
  pass
 def __getslice__(self,*args):
  """ __getslice__(self: str,x: int,y: int) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __mod__(self,*args):
  """ x.__mod__(y) <==> x%y """
  pass
 def __mul__(self,*args):
  """ x.__mul__(y) <==> x*yx.__mul__(y) <==> x*yx.__mul__(y) <==> x*y """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,object: float) -> object

  __new__(cls: type,object: bool) -> object

  __new__(cls: type,object: int) -> object

  __new__(cls: type,string: object,encoding: str,errors: str) -> object

  __new__(cls: type,object: Single) -> object

  __new__(cls: type,object: Extensible[float]) -> object

  __new__(cls: type,object: Extensible[long]) -> object

  __new__(cls: type,object: str) -> object

  __new__(cls: type,object: object) -> object

  __new__(cls: type) -> object

  __new__(cls: type,object: long) -> object

  __new__(cls: type,object: Char) -> object

  __new__(cls: type,object: ExtensibleString) -> object
  """
  pass
 def __ne__(self,*args):
  pass
 def __radd__(self,*args):
  """
  __radd__(self: Char,other: str) -> str

  __radd__(self: str,other: str) -> str
  """
  pass
 def __rmod__(self,*args):
  """ __rmod__(other: object,self: str) -> object """
  pass
 def __rmul__(self,*args):
  """
  __rmul__(count: object,self: str) -> object

  __rmul__(count: Index,self: str) -> object

  __rmul__(other: int,self: str) -> str
  """
  pass



"""ApiDocs API wrapper."""
import json
from collections import namedtuple

from pyrevit import HOST_APP
from pyrevit import coreutils


APIDOCS_INDEX = r'https://static.apidocs.co/apidocs_index.json'
APIDOCS_LOOKUP_TEMPLATE = \
    'https://api.apidocs.co/resolve/{app_name}/{app_version}/'\
        '?asset_id={asset_type}:{asset_id}'


APIDocsApp = namedtuple('APIDocsApp', ['apptitle', 'appslug', 'versionslug'])


def get_apps():
    """Returns a list of application available on apidocs.

    Returns:
        (list[APIDocsApp]): applictations documented on apidocs
    """
    apidoc_apps = []
    for app in json.loads(coreutils.read_url(APIDOCS_INDEX)):
        apidoc_apps.append(
            APIDocsApp(
                apptitle=app['apptitle'],
                appslug=app['appslug'],
                versionslug=app['versionslug']
            )
        )
    return apidoc_apps


def _make_uri(asset_type, asset_id, app_name, app_version):
    """Returns a URI for the given asset.

    Args:
        asset_type (str): Asset type
        asset_id (str): Asset ID
        app_name (str): Application name
        app_version (str): Application version

    Returns:
        (str): URI
    """
    return APIDOCS_LOOKUP_TEMPLATE.format(
        asset_type=asset_type,
        asset_id=asset_id,
        app_name=app_name,
        app_version=app_version,
        )


def make_namespace_uri(namespace,
                       app_name="revit",
                       app_version=str(HOST_APP.version)):
    """Returns the URI of a namespace.

    Args:
        namespace (str): name of the namespace
        app_name (str, optional): name of the application. Defaults to "revit".
        app_version (str, optional): version of the application. Defaults to str(HOST_APP.version).

    Returns:
        (str): URI of the namespace
    """
    return _make_uri(
        asset_type="N",
        asset_id=namespace,
        app_name=app_name,
        app_version=app_version,
        )


def make_type_uri(type_name,
                  app_name="revit",
                  app_version=str(HOST_APP.version)):
    """Returns the URI of a type.

    Args:
        type_name (str): name of the type
        app_name (str, optional): name of the application. Defaults to "revit".
        app_version (str, optional): version of the application. Defaults to str(HOST_APP.version).

    Returns:
        (str): URI of the type
    """
    return _make_uri(
        asset_type="T",
        asset_id=type_name,
        app_name=app_name,
        app_version=app_version,
        )


def make_event_uri(event_name,
                   app_name="revit",
                   app_version=str(HOST_APP.version)):
    """Returns the URI of an event.

    Args:
        event_name (str): name of the event.
        app_name (str): name of the application. Defaults to "revit".
        app_version (str): version of the application.
        

    Returns:
        (str): URI of the event
    """
    return _make_uri(
        asset_type="E",
        asset_id=event_name,
        app_name=app_name,
        app_version=app_version,
        )



class ISupportInitializeNotification(ISupportInitialize):
 """ Allows coordination of initialization for a component and its dependent properties. """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 IsInitialized=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component is initialized.



Get: IsInitialized(self: ISupportInitializeNotification) -> bool



"""


 Initialized=None




class HtmlElement(object):
 """ Represents an HTML element inside of a Web page. """
 def AppendChild(self,newElement):
  """
  AppendChild(self: HtmlElement,newElement: HtmlElement) -> HtmlElement

  

   Adds an element to another element's subtree.

  

   newElement: The System.Windows.Forms.HtmlElement to append to this location in the tree.

   Returns: The element after it has been added to the tree.
  """
  pass
 def AttachEventHandler(self,eventName,eventHandler):
  """
  AttachEventHandler(self: HtmlElement,eventName: str,eventHandler: EventHandler)

   Adds an event handler for a named event on the HTML Document Object Model (DOM).

  

   eventName: The name of the event you want to handle.

   eventHandler: The managed code that handles the event.
  """
  pass
 def DetachEventHandler(self,eventName,eventHandler):
  """
  DetachEventHandler(self: HtmlElement,eventName: str,eventHandler: EventHandler)

   Removes an event handler from a named event on the HTML Document Object Model (DOM).

  

   eventName: The name of the event you want to handle.

   eventHandler: The managed code that handles the event.
  """
  pass
 def Equals(self,obj):
  """
  Equals(self: HtmlElement,obj: object) -> bool

  

   Tests if the supplied object is equal to the current element.

  

   obj: The object to test for equality.

   Returns: true if obj is an System.Windows.Forms.HtmlElement; otherwise,false.
  """
  pass
 def Focus(self):
  """
  Focus(self: HtmlElement)

   Puts user input focus on the current element.
  """
  pass
 def GetAttribute(self,attributeName):
  """
  GetAttribute(self: HtmlElement,attributeName: str) -> str

  

   Retrieves the value of the named attribute on the element.

  

   attributeName: The name of the attribute. This argument is case-insensitive.

   Returns: The value of this attribute on the element,as a System.String value. If the specified attribute 

    does not exist on this element,returns an empty string.
  """
  pass
 def GetElementsByTagName(self,tagName):
  """
  GetElementsByTagName(self: HtmlElement,tagName: str) -> HtmlElementCollection

  

   Retrieves a collection of elements represented in HTML by the specified HTML tag.

  

   tagName: The name of the tag whose System.Windows.Forms.HtmlElement objects you wish to retrieve.

   Returns: An System.Windows.Forms.HtmlElementCollection containing all elements whose HTML tag name is 

    equal to tagName.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: HtmlElement) -> int

   Returns: A hash code for the current System.Object.
  """
  pass
 def InsertAdjacentElement(self,orient,newElement):
  """
  InsertAdjacentElement(self: HtmlElement,orient: HtmlElementInsertionOrientation,newElement: HtmlElement) -> HtmlElement

  

   Insert a new element into the Document Object Model (DOM).

  

   orient: Where to insert this element in relation to the current element.

   newElement: The new element to insert.

   Returns: The System.Windows.Forms.HtmlElement that was just inserted. If insertion failed,this will 

    return null.
  """
  pass
 def InvokeMember(self,methodName,parameter=None):
  """
  InvokeMember(self: HtmlElement,methodName: str,*parameter: Array[object]) -> object

  

   Executes a function defined in the current HTML page by a scripting language.

  

   methodName: The name of the property or method to invoke.

   parameter: A list of parameters to pass.

   Returns: The element returned by the function,represented as an System.Object. If this System.Object is 

    another HTML element,and you have a reference to the unmanaged MSHTML library added to your 

    project,you can cast it to its appropriate unmanaged interface.

  

  InvokeMember(self: HtmlElement,methodName: str) -> object

  

   Executes an unexposed method on the underlying DOM element of this element.

  

   methodName: The name of the property or method to invoke.

   Returns: The element returned by this method,represented as an System.Object. If this System.Object is 

    another HTML element,and you have a reference to the unmanaged MSHTML library added to your 

    project,you can cast it to its appropriate unmanaged interface.
  """
  pass
 def RaiseEvent(self,eventName):
  """
  RaiseEvent(self: HtmlElement,eventName: str)

   Causes the named event to call all registered event handlers.

  

   eventName: The name of the event to raise.
  """
  pass
 def RemoveFocus(self):
  """
  RemoveFocus(self: HtmlElement)

   Removes focus from the current element,if that element has focus.
  """
  pass
 def ScrollIntoView(self,alignWithTop):
  """
  ScrollIntoView(self: HtmlElement,alignWithTop: bool)

   Scrolls through the document containing this element until the top or bottom edge of this 

    element is aligned with the document's window.

  

  

   alignWithTop: If true,the top of the object will be displayed at the top of the window. If false,the bottom 

    of the object will be displayed at the bottom of the window.
  """
  pass
 def SetAttribute(self,attributeName,value):
  """
  SetAttribute(self: HtmlElement,attributeName: str,value: str)

   Sets the value of the named attribute on the element.

  

   attributeName: The name of the attribute to set.

   value: The new value of this attribute.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __ne__(self,*args):
  pass
 All=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an System.Windows.Forms.HtmlElementCollection of all elements underneath the current element.



Get: All(self: HtmlElement) -> HtmlElementCollection



"""

 CanHaveChildren=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether this element can have child elements.



Get: CanHaveChildren(self: HtmlElement) -> bool



"""

 Children=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an System.Windows.Forms.HtmlElementCollection of all children of the current element.



Get: Children(self: HtmlElement) -> HtmlElementCollection



"""

 ClientRectangle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the bounds of the client area of the element in the HTML document.



Get: ClientRectangle(self: HtmlElement) -> Rectangle



"""

 Document=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.HtmlDocument to which this element belongs.



Get: Document(self: HtmlElement) -> HtmlDocument



"""

 DomElement=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an unmanaged interface pointer for this element.



Get: DomElement(self: HtmlElement) -> object



"""

 Enabled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets whether the user can input data into this element.



Get: Enabled(self: HtmlElement) -> bool



Set: Enabled(self: HtmlElement)=value

"""

 FirstChild=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the next element below this element in the document tree.



Get: FirstChild(self: HtmlElement) -> HtmlElement



"""

 Id=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a label by which to identify the element.



Get: Id(self: HtmlElement) -> str



Set: Id(self: HtmlElement)=value

"""

 InnerHtml=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the HTML markup underneath this element.



Get: InnerHtml(self: HtmlElement) -> str



Set: InnerHtml(self: HtmlElement)=value

"""

 InnerText=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the text assigned to the element.



Get: InnerText(self: HtmlElement) -> str



Set: InnerText(self: HtmlElement)=value

"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the element.



Get: Name(self: HtmlElement) -> str



Set: Name(self: HtmlElement)=value

"""

 NextSibling=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the next element at the same level as this element in the document tree.



Get: NextSibling(self: HtmlElement) -> HtmlElement



"""

 OffsetParent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the element from which System.Windows.Forms.HtmlElement.OffsetRectangle is calculated.



Get: OffsetParent(self: HtmlElement) -> HtmlElement



"""

 OffsetRectangle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the location of an element relative to its parent.



Get: OffsetRectangle(self: HtmlElement) -> Rectangle



"""

 OuterHtml=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the current element's HTML code.



Get: OuterHtml(self: HtmlElement) -> str



Set: OuterHtml(self: HtmlElement)=value

"""

 OuterText=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the current element's text.



Get: OuterText(self: HtmlElement) -> str



Set: OuterText(self: HtmlElement)=value

"""

 Parent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the current element's parent element.



Get: Parent(self: HtmlElement) -> HtmlElement



"""

 ScrollLeft=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the distance between the edge of the element and the left edge of its content.



Get: ScrollLeft(self: HtmlElement) -> int



Set: ScrollLeft(self: HtmlElement)=value

"""

 ScrollRectangle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the dimensions of an element's scrollable region.



Get: ScrollRectangle(self: HtmlElement) -> Rectangle



"""

 ScrollTop=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the distance between the edge of the element and the top edge of its content.



Get: ScrollTop(self: HtmlElement) -> int



Set: ScrollTop(self: HtmlElement)=value

"""

 Style=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a comma-delimited list of styles for the current element.



Get: Style(self: HtmlElement) -> str



Set: Style(self: HtmlElement)=value

"""

 TabIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the location of this element in the tab order.



Get: TabIndex(self: HtmlElement) -> Int16



Set: TabIndex(self: HtmlElement)=value

"""

 TagName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the HTML tag.



Get: TagName(self: HtmlElement) -> str



"""


 Click=None
 DoubleClick=None
 Drag=None
 DragEnd=None
 DragLeave=None
 DragOver=None
 Focusing=None
 GotFocus=None
 KeyDown=None
 KeyPress=None
 KeyUp=None
 LosingFocus=None
 LostFocus=None
 MouseDown=None
 MouseEnter=None
 MouseLeave=None
 MouseMove=None
 MouseOver=None
 MouseUp=None




class DataGridViewSelectedCellCollection(BaseCollection,ICollection,IEnumerable,IList):
 """ Represents a collection of cells that are selected in a System.Windows.Forms.DataGridView. """
 def Clear(self):
  """
  Clear(self: DataGridViewSelectedCellCollection)

   Clears the collection.
  """
  pass
 def Contains(self,dataGridViewCell):
  """
  Contains(self: DataGridViewSelectedCellCollection,dataGridViewCell: DataGridViewCell) -> bool

  

   Determines whether the specified cell is contained in the collection.

  

   dataGridViewCell: The System.Windows.Forms.DataGridViewCell to locate in the 

    System.Windows.Forms.DataGridViewSelectedCellCollection.

  

   Returns: true if dataGridViewCell is in the System.Windows.Forms.DataGridViewSelectedCellCollection; 

    otherwise,false.
  """
  pass
 def CopyTo(self,*__args):
  """
  CopyTo(self: DataGridViewSelectedCellCollection,array: Array[DataGridViewCell],index: int)

   Copies the elements of the collection to the specified System.Windows.Forms.DataGridViewCell 

    array,starting at the specified index.

  

  

   array: The one-dimensional array of type System.Windows.Forms.DataGridViewCell that is the destination 

    of the elements copied from the collection. The array must have zero-based indexing.

  

   index: The zero-based index in array at which copying begins.
  """
  pass
 def Insert(self,index,dataGridViewCell):
  """
  Insert(self: DataGridViewSelectedCellCollection,index: int,dataGridViewCell: DataGridViewCell)

   Inserts a cell into the collection.

  

   index: The index at which dataGridViewCell should be inserted.

   dataGridViewCell: The object to be added to the System.Windows.Forms.DataGridViewSelectedCellCollection.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def __contains__(self,*args):
  """
  __contains__(self: IList,value: object) -> bool

  

   Determines whether the System.Collections.IList contains a specific value.

  

   value: The object to locate in the System.Collections.IList.

   Returns: true if the System.Object is found in the System.Collections.IList; otherwise,false.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 List=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a list of elements in the collection.



"""





class CounterSampleCalculator(object):
 """ Provides a set of utility functions for interpreting performance counter data. """
 @staticmethod
 def ComputeCounterValue(*__args):
  """
  ComputeCounterValue(oldSample: CounterSample,newSample: CounterSample) -> Single

  

   Computes the calculated value of two raw counter samples.

  

   oldSample: A System.Diagnostics.CounterSample that indicates a previous sample the system has taken.

   newSample: A System.Diagnostics.CounterSample that indicates the most recent sample the system has taken.

   Returns: A floating-point representation of the performance counter's calculated value.

  ComputeCounterValue(newSample: CounterSample) -> Single

  

   Computes the calculated value of a single raw counter sample.

  

   newSample: A System.Diagnostics.CounterSample that indicates the most recent sample the system has taken.

   Returns: A floating-point representation of the performance counter's calculated value.
  """
  pass
 __all__=[
  'ComputeCounterValue',
 ]




class StringConverter(TypeConverter):
 """
 Provides a type converter to convert string objects to and from other representations.

 

 StringConverter()
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: StringConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Gets a value indicating whether this converter can convert an object in the given source type to 

    a string using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   sourceType: A System.Type that represents the type you wish to convert from.

   Returns: true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: StringConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts the specified value object to a System.String object.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: The System.Globalization.CultureInfo to use.

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.
  """
  pass



# encoding: utf-8
# module System.Windows.Input.StylusWisp calls itself StylusWisp
# from PresentationCore,Version=4.0.0.0,Culture=neutral,PublicKeyToken=31bf3856ad364e35
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class WispTabletDeviceCollection(TabletDeviceCollection,ICollection,IEnumerable):
 # no doc
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass




class EventLogPermissionEntry(object):
 """
 Defines the smallest unit of a code access security permission that is set for an System.Diagnostics.EventLog.

 

 EventLogPermissionEntry(permissionAccess: EventLogPermissionAccess,machineName: str)
 """
 @staticmethod
 def __new__(self,permissionAccess,machineName):
  """ __new__(cls: type,permissionAccess: EventLogPermissionAccess,machineName: str) """
  pass
 MachineName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the computer on which to read or write events.



Get: MachineName(self: EventLogPermissionEntry) -> str



"""

 PermissionAccess=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the permission access levels used in the permissions request.



Get: PermissionAccess(self: EventLogPermissionEntry) -> EventLogPermissionAccess



"""





class SplitterCancelEventArgs(CancelEventArgs):
 """
 Provides data for splitter events.

 

 SplitterCancelEventArgs(mouseCursorX: int,mouseCursorY: int,splitX: int,splitY: int)
 """
 @staticmethod
 def __new__(self,mouseCursorX,mouseCursorY,splitX,splitY):
  """ __new__(cls: type,mouseCursorX: int,mouseCursorY: int,splitX: int,splitY: int) """
  pass
 MouseCursorX=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the X coordinate of the mouse pointer in client coordinates.



Get: MouseCursorX(self: SplitterCancelEventArgs) -> int



"""

 MouseCursorY=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the Y coordinate of the mouse pointer in client coordinates.



Get: MouseCursorY(self: SplitterCancelEventArgs) -> int



"""

 SplitX=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the X coordinate of the upper left corner of the System.Windows.Forms.SplitContainer in client coordinates.



Get: SplitX(self: SplitterCancelEventArgs) -> int



Set: SplitX(self: SplitterCancelEventArgs)=value

"""

 SplitY=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the Y coordinate of the upper left corner of the System.Windows.Forms.SplitContainer in client coordinates.



Get: SplitY(self: SplitterCancelEventArgs) -> int



Set: SplitY(self: SplitterCancelEventArgs)=value

"""





class AlternationConverter(object,IValueConverter):
 """
 Converts an integer to and from an object by applying the integer as an index to a list of objects.

 

 AlternationConverter()
 """
 def Convert(self,o,targetType,parameter,culture):
  """
  Convert(self: AlternationConverter,o: object,targetType: Type,parameter: object,culture: CultureInfo) -> object

  

   Converts an integer to an object in the System.Windows.Controls.AlternationConverter.Values list.

  

   o: The integer to use to find an object in the System.Windows.Controls.AlternationConverter.Values 

    property.

  

   targetType: The type of the binding target property.

   parameter: The converter parameter to use.

   culture: The culture to use in the converter.

   Returns: The object that is in the position of o modulo the number of items in 

    System.Windows.Controls.AlternationConverter.Values.
  """
  pass
 def ConvertBack(self,o,targetType,parameter,culture):
  """
  ConvertBack(self: AlternationConverter,o: object,targetType: Type,parameter: object,culture: CultureInfo) -> object

  

   Converts an object in the System.Windows.Controls.AlternationConverter.Values list to an integer.

  

   o: The object to find in the System.Windows.Controls.AlternationConverter.Values property.

   targetType: The type of the binding target property.

   parameter: The converter parameter to use.

   culture: The culture to use in the converter.

      not exist in System.Windows.Controls.AlternationConverter.Values.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Values=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a list of objects that the System.Windows.Controls.AlternationConverter returns when an integer is passed to the System.Windows.Controls.AlternationConverter.Convert(System.Object,System.Type,System.Object,System.Globalization.CultureInfo) method.



Get: Values(self: AlternationConverter) -> IList



"""





class StandardOleMarshalObject(MarshalByRefObject,IMarshal):
 """ Replaces the standard common language runtime (CLR) free-threaded marshaler with the standard OLE STA marshaler. """
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class AddingNewEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.BindingSource.AddingNew event.

 

 AddingNewEventArgs()

 AddingNewEventArgs(newObject: object)
 """
 @staticmethod
 def __new__(self,newObject=None):
  """
  __new__(cls: type)

  __new__(cls: type,newObject: object)
  """
  pass
 NewObject=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the object to be added to the binding list.



Get: NewObject(self: AddingNewEventArgs) -> object



Set: NewObject(self: AddingNewEventArgs)=value

"""


